; es23_0800.bin
; JU+TE TINY ES2.3 Harun Scheutzow 1989
; reass. V. Pohlers 01.2022

		cpu	z8601
		assume RP:0C0h		; prevent register optimization thru AS
		
; AS-Funktionen
hi	function x,(x>>8)&255
lo	function x, x&255

;------------------------------------------------------------------------------
; Register
;------------------------------------------------------------------------------

word_4 equ 4				; wr_block:wr_block1w
byte_6 equ 6
byte_7 equ 7
byte_8 equ 8
byte_9 equ 9
byte_B equ 0Bh				; sub_1494:loc_149Fw ...
byte_D equ 0Dh
byte_E equ 0Eh
byte_F equ 0Fh
word_10	equ 10h
byte_12	equ 12h
byte_13	equ 13h
byte_14	equ 14h
byte_15	equ 15h
byte_16	equ 16h				; Flagregister des Nutzers
byte_17	equ 17h				; Registerpointer des Nutzers
byte_18	equ 18h				; Zahl zum Ausdrucken
byte_19	equ 19h
word_1A	equ 1Ah				; Zeiger auf das bearbeitete byte im Speicher
byte_1C	equ 1Ch				; Ergebnis der ASCII->Zahl Wandlung
byte_1D	equ 1Dh
byte_1E	equ 1Eh				; Zeiger auf das nchste byte im ASCII-Zeilenpuffer
;
word_20	equ 20h				; VAR A
word_22	equ 22h				; VAR B
byte_24	equ 24h				; VAR C
var_V_lo equ 4Bh
var_W_lo equ 4Dh
var_X_lo equ 4Fh
var_Y_lo equ 51h
var_Z_lo equ 53h
;
byte_54	equ 54h				; IRQ-Zhler
byte_55	equ 55h				; maximale logische Zeilenlnge: %10,%20 oder %40
irq4vec_hi	equ 56h			; BildIRQvektor
irq4vec_lo 	equ 57h
byte_58	equ 58h				; Zeilenlnge
byte_5A	equ 5Ah
cupos		equ 5Bh			; Kursorposition (absolut ab F400)
byte_5C	equ 5Ch
byte_5D	equ 5Dh				; Drucker an/aus
byte_5E	equ 5Eh
byte_5F	equ 5Fh
byte_60	equ 60h
byte_61	equ 61h
byte_62	equ 62h
byte_63	equ 63h
byte_66	equ 66h
byte_67	equ 67h				; hi (Zeichengenerator)
byte_6C	equ 6Ch				; Autorepeatflag
byte_6D	equ 6Dh				; gedr.	Taste
unk_6E equ 6Eh
byte_6F	equ 6Fh
word_72	equ 72h

; Speicherbelegung
; 
; (%F400-F4FF	ASCII-Textspeicher des FSE)
; %F500-F57A		System-RAM
; (%F57B-F5FF	Kassettenpuffer)
; %F600-F6FF		Stack
; %F700-F73F		Zeilenspeicher bei Rckkehr aus dem FSE (Routine CHARIN)
; $F740-F77F		CONTROL-Belegung der Tasten
; $F780-F78F		hier speichert MON Routine MONITOR) die Inhalte der Register %10-1F
; %F790-F79F		belegt durch SAVE und LOAD zum Zwischenspeichern der Register %04-0F
; %F7A0-F7FF		noch frei
; ($F800-FFFF	Bildwiederholspeicher)
; 
; Blockaufbau: (Adresse im Kassettenpuffer als erste Zahl)
; %F57B		Blocknummer %00 bis %FF, evt. Wieder von vorn
; %F57C		Kopie von %F57B
; %F57D		Prfsumme ber Formatbyte und Datenbytes (8 bit Summe mit dazuaddierten Carrys)
; %F57E		Kopie von %F57D
; %F57F		Formatbyte:	%FA	weniger als 128 Datenbyte, Anzahl
; auf %F5FF
; 					%FC	128 Datenbyte
; 					%FE	End of File Block, ohne sinnvolle
; Datenbytes
; %F580-F5FF	Datenbytes


;------------------------------------------------------------------------------
; System-Register
;------------------------------------------------------------------------------

p0 equ	0				; Port 0
p1 equ	1				; Port 1
p2 equ	2				; Port 2
p3 equ	3				; Port 3

sio equ 0F0h				; Serial I/O
tmr equ 0F1h				; Timer	mode
t1 equ	0F2h				; Timer/counter	1
pre1 equ 0F3h				; T1 prescaler
t0 equ	0F4h				; Timer/counter	0
pre0 equ 0F5h				; T0 prescaler
p2m equ 0F6h				; Port 2 mode
p3m equ 0F7h				; Port 3 mode
p01m equ 0F8h				; Ports	0-1 mode
ipr equ 0F9h				; Interrupt priority register
irq equ 0FAh				; Interrupt request register
imr equ 0FBh				; Interrupt mask register
flags equ 0FCh				; Program control flags
rp equ	0FDh				; Register pointer
sph equ 0FEh				; Stack	Pointer
spl equ 0FFh	

;------------------------------------------------------------------------------
; 
;------------------------------------------------------------------------------

		org 800h

		jp	0F500h		; IRQ 0
		jp	0F503h		; IRQ 1
		jp	0F506h		; IRQ 2
		jp	0F509h		; IRQ 3
		jp	@irq4vec_hi	; BildIRQvektor
		nop
		jp	0F50Fh		; IRQ 5
		jp	KOMMAND		; Eintritt aus Bootstrap Module
		;
j_CHARIN:	jp	CHARIN
j_CHAROUT:	jp	CHAROUT
j_KEY:		jp	KEY
j_WKEY:		jp	WKEY
j_SAVE:		jp	SAVE
j_LOAD:		jp	LOAD
		jp	SCRFUN
j_MONITOR:	jp	MONITOR
		jp	0FFFFh		; frei f. Erweiterung

;------------------------------------------------------------------------------
; Bilddarstellung
; IRQ4_Handler 1
;------------------------------------------------------------------------------

p_irq1:		or	p3, #80h	; P37 setzen
		and	p3, #7Fh	; P37 lschen, Synchronimpuls
		dec	byte_54		; IRQ-Zhler
		jr	Z, loc_83B
		iret			; wir sind noch im Strahlrcklauf
; Beginn des sichtbaren Bereiches
loc_83B:	ld	irq4vec_hi, sph	; Stackpointer sichern
		ld	irq4vec_lo, spl 
		incw	unk_6E
		ld	sph, #0F4h	; Stackpointer auf Beginn des ASCII-Textspeichers F400
		clr	spl		; 
		ld	byte_54, #8	; 
loc_84B:	dec	byte_54		; 
		jr	NZ, loc_84B
		JR 	F,$+2		; nop
		ld	byte_54, #0C0h	; 192 Zeilen
		; 2. HSync-Impuls
loc_854:	or	p3, #80h	; P37 setzen
		and	p3, #7Fh	; P37 lschen, Horizontalsynchronimpuls
		; 3. Schwarzschulter
		tm	byte_6D, #0FFh	; gedr.	Taste
		jr	Z, loc_864	; nein
		xor	p3, #40h	; sonst P36 toggeln
		jr	loc_869
		; 4. sichtbarer Teil, 16 Byte darstellen
loc_864:	and	p3, #3Fh	; Port 3
		JR 	F,$+2		; nop
loc_869:	JR 	F,$+2		; nop
		nop
		pop	80h		; 16 Byte ausgeben
		pop	80h
		pop	80h
		pop	80h
		pop	80h
		pop	80h
		pop	80h
		pop	80h
		pop	80h
		pop	80h
		pop	80h
		pop	80h
		pop	80h
		pop	80h
		pop	80h
		pop	80h
		dec	byte_54		; Zeilen-Zhler
		jr	NZ, loc_854
		;
		or	p3, #80h	; HSync ein
		and	p3, #7Fh	; HSync aus
		ld	byte_54, #51h	; IRQ-Zhler, 81 Zeilen
		ld	sph, irq4vec_hi	; Stackpointer wiederherstellen
		ld	spl, irq4vec_lo
		ld	irq4vec_hi, #hi(p_irq2)	; weiter mit p_irq2
		ld	irq4vec_lo, #lo(p_irq2)
		and	irq, #0EFh	; IRQ4 lschen
		iret

;------------------------------------------------------------------------------
; Bilddarstellung
; IRQ4_Handler 2
; unterer Bildrand
;------------------------------------------------------------------------------

p_irq2:		or	p3, #80h	; HSync ein
		and	p3, #7Fh	; HSync aus
		dec	byte_54		; IRQ-Zhler
		jr	Z, loc_8B4
		iret
;
loc_8B4:	ld	byte_54, #6	; IRQ-Zhler
		ld	irq4vec_lo, #lo(p_irq3)	; weiter mit p_irq3
		iret

;------------------------------------------------------------------------------
; Bilddarstellung
; IRQ4_Handler 3
; Strahlrcklauf
;------------------------------------------------------------------------------

p_irq3:		or	p3, #80h	; HSync ein
		dec	byte_54		; IRQ-Zhler
		jr	Z, loc_8C3
		iret
;
loc_8C3:	and	p3, #7Fh	; HSync aus
		ld	byte_54, #20h	; IRQ-Zhler die letzten 32 Zeilen
		ld	irq4vec_lo, #lo(p_irq1)	; weiter mit p_irq1
		iret

;------------------------------------------------------------------------------
; ASCII-Cursorpos-> RR4 = graph. BWS-Adr 
;------------------------------------------------------------------------------

sub_8CD:	ld	R4, #0F0h
		and	R4, cupos	; Kursorposition
		swap	R4		
		ld	R5, #0Fh
		and	R5, cupos	; Kursorposition
		rl	R5
		sra	R4
		rrc	R5
		add	R4, #0F8h	; F800 Bildwiederholspeicher
		ret

		db 0FFh

;------------------------------------------------------------------------------
; Zeichenausgabe in ASCII-Speicher und graph. BWS
; in: cupos Cursorposition, byte_5A Zeichen, byte_67 Zeichensatztabelle
;------------------------------------------------------------------------------

sub_8E4:	push	rp		; Register pointer
		srp	#60h
		ld	R0, #0F4h	; F400 ASCII-Textspeicher
		ld	R1, cupos	; Kursorposition
		ld	R2, byte_5A
		lde	@RR0, R2	; schreiben in ASCII-Speicher
		call	sub_8CD		; ASCII Cursorpos -> graph. BWS-Adr 
		ld	R3, #0
		ld	R1, #5		; 5x verschieben nach rechts
loc_8F7:	rcf
		rrc	R2
		rrc	R3
		djnz	R1, loc_8F7	; RR2 = Zeichen*8
		add	R2, R7		; RR2=Pos in Zeichensatztabelle
		ld	R1, #8		; 8 Zeilen pro Zeichen
loc_902:	ldc	R0, @RR2
		lde	@RR4, R0	; schreiben in graph. BWS
		add	R5, #10h	; nchste Pos. in BWS
		inc	R3		; nchstes Byte des Zeichens
		djnz	R1, loc_902
		pop	rp		; Register pointer
		ret
		
		db 0FFh

;------------------------------------------------------------------------------
; init Systemregister
;------------------------------------------------------------------------------

sub_910:	srp	#0F0h
		ld	R11, #10h	; imr	IRQ4 erlauben, INTs aus
		ld	R9, #8		; ipr	IRQ4 auf niedrigste Prioritt
		ld	R8, #92h	; p01m	P0/P1 Adressbus, Stack extern, Timing normal
		ld	R7, #8		; p3m
		ld	R5, #5		; pre0	Prescaler =1, Timer modulo N
		ld	R4, #40h	; t0	Timer 0 auf 64 (d.h. INT4 alle 64s bzw 15,625kHz)
		ld	R1, #3		; tmr	Timer 0 laden und starten
		nop
		nop
		nop
		nop
		ei
		ret

		db 0FFh
		db 0FFh

;------------------------------------------------------------------------------
; Einsprung nach Reset
;------------------------------------------------------------------------------

KOMMAND:	srp	#0F0h
		ld	R14, #0F7h	; sph	SP = F700
		ld	R15, #0		; spl
		ld	R8, #92h	; p01m	P0/P1 Adressbus, Stack extern, Timing normal
		;
		ld	byte_6,	#0E0h	; BASIC-Programmstart auf 0E000h
		clr	byte_7
		clr	byte_8
		;
		srp	#50h
		ld	R6, #hi(p_irq1)
		ld	R7, #lo(p_irq1)
		ld	R13, #0
		ld	byte_67, #0Fh	; Std-Zeichensatztabelle
		ld	R5, #40h
		call	sub_910		; init Systemregister
		call	sub_AE0		; Init Msg ausgeben
		nop
		nop
		nop
		nop

kdoloop:	ld	byte_55, #40h	; log. Zeilenlnge
		ld	byte_58, #0FFh	; Zeilenlnge
		call	kdo13
		srp	#10h
		call	j_CHARIN
		cp	byte_13, #0Dh
		jr	Z, kdoloop
		ld	R6, byte_13
;
;?X	druckt Wert der Variable X aus 		
		cp	R6, #'?'
		jr	NZ, kdo2
		call	j_CHARIN	; 2. Zeichen
		ld	R5, byte_13
		ld	R6, byte_13
		rl	byte_16		; R6*2
		ld	R2, 9Eh(R6)	; A=41, *2=82, +9E=120 = 20h
		ld	R3, 9Fh(R6)	; RR2 = Adr. Variable
		call	j_CHAROUT	; Ausgabe Variable
		ld	R5, #3Dh	; '='
		call	j_CHAROUT
kdo1:		call	PrDez		; Ausgabe Wert
		ld	R5, #0Dh
		call	j_CHAROUT
		jr	kdoloop
;
; CONT setzt das Programm nach STOP-Befehl fort
kdo2:		cp	R6, #'C'
		jr	NZ, kdo3
		tm	byte_F,	#8	; sind wir auf STOP?
		jr	Z, kdo7		; nein 
		and	byte_F,	#0F7h	; STOP-Flag lschen
		jr	kdo6		; und Programm fortsetzen
;
kdo3:		ld	byte_F,	#4
		clr	byte_E		; Fehlerflags lschen
		ld	R0, byte_6	; Pointer auf Anfang des BASIC-Programmes
		ld	R1, byte_7
;
; N	New, lscht das	Basic-Programm
		cp	R6, #'N'
		jr	NZ, kdo5
		ld	R4, #0
		lde	@RR0, R4	; erstes Byte des BASIC-Programmes auf 0 setzen
kdo4:		jr	kdoloop
;
; RUN	startet das Programm
kdo5:		cp	R6, #'R'
		jr	NZ, kdo9
		call	6C9h		; U883BAS: c_CLRTRAP
					; Variablen lschen und	BASIC initialisieren
kdo6:		clr	byte_6D		; gedr.	Taste
		ld	byte_55, #10h	; log. Zeilenlnge
		ld	byte_58, #0FFh	; Zeilenlnge
		call	738h		; U883BAS: run1b, Programm fortsetzen
		ld	R5, #0Dh
		call	j_CHAROUT
		ld	R5, #'O'	; Ok, Ende mit END
		tm	byte_F,	#2	; sind wir fertig?
		jr	NZ, kdo8	; ok --> weiter
		ld	R5, #'S'	; Ende mit STOP
		tm	byte_F,	#8	; sind wir auf STOP?
		jr	NZ, kdo8	; ok --> weiter
kdo7:		ld	R5, #'E' 	; Sonst Error
kdo8:		call	j_CHAROUT
		ld	R3, #0F0h
		and	byte_13, byte_E
		swap	byte_13
		clr	byte_12
		jr	kdo1
;
; E	List
kdo9:		cp	R6, #'E'
		jp	Z, list
;		
; P, PO	Drucker
		cp	R6, #'P'
		jr	NZ, kdo10
		call	j_CHARIN
		ld	byte_5D, R3	; Drucker an/aus
		jr	kdo4

kdo10:		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
;
; S	speichert ein Programm auf Kassette
		cp	R6, #'S'
		jr	NZ, kdo12
		call	sub_EC5
		ld	word_20, R0
		ld	word_20+1, R1
		sub	R3, R1
		sbc	R2, R0
		incw	RR2
		ld	word_22, R2
		ld	word_22+1, R3
		call	j_SAVE
kdo11:		jp	kdoloop
;
; L	ldt ein Programm von Kassette
kdo12:		cp	R6, #'L'
		jp	NZ, kdo14
		ld	word_20, R0
		ld	word_20+1, R1
		call	j_LOAD
		jr	kdo11
kdo13:		ld	byte_6C, #0	; Autorepeatflag
		ld	byte_15, #6
		call	j_CHAROUT
		ret
;
; M	ruft den MON auf
kdo14:		cp	R6, #'M'
		jp	NZ, kdo15
		call	j_MONITOR
		jp	kdoloop

		db 0FFh

;------------------------------------------------------------------------------
; wandelt eine 5 Ziffern lange Dezimalzahl (00000 bis 65535) ab (01Eh/1F) 
; nach 014h/15, 012h/13 und 01Ch/1D zerstrt, 01Fh um fnf erhht
;------------------------------------------------------------------------------

DA5TH16:	ld	R12, #5
		ld	R4, #0
		ld	R5, #0
da1:		ld	R2, #0
		ld	R3, #0
		ld	R13, #0Ah
da2:		add	R3, R5
		adc	R2, R4
		djnz	R13, da2
		ld	R5, byte_13
		ld	R4, byte_12
		lde	R13, @RR14
		sub	byte_1D, #30h
		jr	C, da3
		cp	byte_1D, #0Ah
		jr	NC, da3
		add	R5, R13
		adc	byte_14, #0
		inc	R15
		djnz	R12, da1
		rcf
		ret
da3:		scf
		ret

;------------------------------------------------------------------------------
; E	listet das Programm beginnend mit der ersten Zeile zeilenweise auf, 
; zur Fortsetzung Leertaste drcken, sonst eine andere Taste
; E00125 beginnt mit dem Auflisten bei Zeile 00125, sonst wie E ohne folgende Zahl
;------------------------------------------------------------------------------

list:		ld	R15, #1		; F701 Beginn Zahl
		ld	R14, #0F7h	; F700 Zeilenspeicher 
		call	DA5TH16
		or	R4, #80h	; Hi Bit setzen
		; suche Zeile
list1:		lde	R10, @RR0	; Zeilennummer
		incw	RR0		; aus Basic-Programm
		lde	R11, @RR0
		decw	RR0
		or	R10, R10
		jr	Z, list4	; Basic-Ende
		cp	R10, R4
		jr	C, list2
		jr	UGT, list5	; aktuelle Zeile > Zeile
		cp	R11, R5
		jr	Z, list5	; aktuelle Zeile = Zeile
		jr	UGT, list5	; aktuelle Zeile > Zeile
list2:		incw	RR0
list3:		incw	RR0
		lde	R10, @RR0
		cp	R10, #0Dh	; Zeilenende?
		jr	NZ, list3
		incw	RR0		; Zeilenende berspringen
		jr	list1		; nchste Zeile
; Ende
list4:		jp	kdoloop
; Anzeige ab >= Zeile
list5:		lde	R2, @RR0
		tm	R2, #80h
		jr	Z, list4	; keine Zeilennummer -> Ende
		incw	RR0
		lde	R3, @RR0
		and	R2, #7Fh	; hi Bit rcksetzen
		call	PrDez		; Zeilennummer ausgeben
list6:		incw	RR0
		lde	R5, @RR0
		call	j_CHAROUT	; einzelne Zeichen ausgeben
		cp	R5, #0Dh	; Zeilenende?
		jr	NZ, list6	; nein, weiter
		call	j_WKEY		; ja, Tastatur abfragen
		cp	R3, #' '	; Leertaste gedrckt?
		jr	NZ, list4	; nein -> Ende
		incw	RR0
		jr	list5		; sonst weiter listen

		db 0FFh
		db 0FFh
		db 0FFh

;------------------------------------------------------------------------------
; Ausgabe Systemmeldung
;------------------------------------------------------------------------------
		
sub_AE0:	srp	#10h
		ld	R0, #0Ah	; aInitMsg
		ld	R1, #-16	; Anz Zeichen
loc_AE6:	lde	R5, @RR0
		call	j_CHAROUT
		inc	R1
		jr	NZ, loc_AE6
		ret
		db 0FFh

;------------------------------------------------------------------------------
; Systemmeldung
;------------------------------------------------------------------------------

aInitMsg:	db 0Ch,"ES2.3_SWB_1989",0Dh ; 16 Zeichen

;------------------------------------------------------------------------------
; frei f. Erweiterungen
;------------------------------------------------------------------------------

		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		
;------------------------------------------------------------------------------
; MONITOR	ruft den Maschinenmonitor MON auf. Rckkehr aus	MON mit	Q.
;------------------------------------------------------------------------------
		
MONITOR:	ld	byte_15, #5
		call	j_CHAROUT
		push	rp		; Register pointer
		;Sichern Register 10..1F
		srp	#60h
		ld	R0, #0F7h	; Zwischenspeicher F780
		ld	R1, #80h
		ld	R2, #10h	; erstes zu sicherndes Register
		ld	R3, #10h	; Anzahl
mon1:		ldei	@RR0, @R2
		djnz	R3, mon1
		;
		srp	#10h
mon2:		ld	byte_58, #0FFh	; Zeilenlnge
		ld	byte_55, #20h	; log. Zeilenlnge
		call	j_CHARIN
		ld	R15, #0
		ld	R14, #0F7h	; F700 Zeilenspeicher
		lde	R0, @RR14
		inc	R15
		ld	R12, #hi(kdotab)
		ld	R13, #lo(kdotab)
		ld	R1, #10h	; 10 Kommandos in kdotab
mon3:		lde	R2, @RR12
		inc	R13
		cp	R2, R0		; vgl. Kdo
		jr	NZ, mon4	; nicht gleich
		lde	R0, @RR12	; gefunden -> hole Adr.
		inc	R13
		lde	R1, @RR12
		call	@word_10	; Monitorfunktion aufrufen
		jr	mon2		; weiter im MON
		;
mon4:		inc	R13
		inc	R13
		djnz	R1, mon3
		jr	mon2

;------------------------------------------------------------------------------
; Befehlseintrge
;------------------------------------------------------------------------------

kdotab:		db  'F'
		dw mon_fill
		db  'P'
		dw mon_printer
		db  'H'
		dw mon_hexanz
		db  ','
		dw mon_hexwrite
		db  'J'
		dw mon_jump
		db  'A'
		dw mon_ascii
		db  ';'
		dw mon_asciiwrite
		db  'S'
		dw mon_save
		db  'L'
		dw mon_load
		db  'M'
		dw mon_move
		db  '?'
		dw mon_arith
		db  'Q'
		dw mon_quit
		db  'D'
		dw mon_disass
		db  '.'
		dw mon_assembl
		db  'E'
		dw mon_extra
		db 0FFh		; frei f. Erweiterung
		db 0FFh
		db 0FFh

;------------------------------------------------------------------------------
; HTA16	druckt die 16 bit aus 018h/19 als 4-Ziffer-Hexzahl aus. 018h/19 zerstrt.
;------------------------------------------------------------------------------

HTA16:		push	R9
		ld	byte_19, R8
		call	HTA8
		pop	R9

;------------------------------------------------------------------------------
; HTA8		druckt die 8 bit aus 019h als 2-Ziffer-Hexzahl aus, 019h zerstrt
;------------------------------------------------------------------------------

HTA8:		push	R9
		swap	R9
		call	HTA4
		pop	R9

;------------------------------------------------------------------------------
; druckt die low-4 bit aus 019h als Hexziffer aus, 019h zerstrt
;------------------------------------------------------------------------------

HTA4:		ld	R5, #0Fh
		and	R5, R9
		add	R5, #30h	; '0'
		cp	R5, #3Ah	; '9'+1
		jr	C, hta4_1
		add	R5, #7
hta4_1:		jp	j_CHAROUT

;------------------------------------------------------------------------------
; druckt einen Return-Kode aus (0Dh)
;------------------------------------------------------------------------------

PRRET:		ld	R5, #0Dh
		jr	hta4_1

;------------------------------------------------------------------------------
; druckt einen Return-Kode aus und wartet dann auf Tastendruck, wurde die
; Leertaste gedrckt dann ist das Z-Flag gesetzt
;------------------------------------------------------------------------------

RWCONT:		call	PRRET
		call	j_WKEY
		cp	byte_13, #' '
		ret

;------------------------------------------------------------------------------
; druckt das Zeichen aus 015h aus, dann die 16 bit aus 01Ah/1B 
; (018h/19 zerstrt) als Hexzahl und noch ein Leerzeichen
;------------------------------------------------------------------------------

PCAS:		call	j_CHAROUT
		ld	byte_18, R10
		ld	byte_19, R11
		call	HTA16
		ld	R5, #' '
		jr	hta4_1

;------------------------------------------------------------------------------
; wandelt die 4 Ziffern ab Adresse (01Eh/1F) nach 01Ch/1D und 01Ah/1B, 
; bei Wandlungsfehler wird POP POP RET ausgefhrt, sonst Rckkehr mit um 
; fnf erhhtem 01Fh
;------------------------------------------------------------------------------

ADRE:		call	ATH16
		jr	NC, adre1
		pop	R0
		pop	R0
adre1:		ld	R10, R12
		ld	R11, R13
		inc	R15
		ret
;------------------------------------------------------------------------------
; Fehlermeldung von den folgenden 4 Routinen: C-Flag gesetzt, sonst gelscht
; Die Erhhung von 01Fh stimmt nur bei gelschtem C-Flag.
;
; wandelt die Ziffer von (01Eh/1F) in low-4 bit von 01Dh, 01Fh um eins erhht
;------------------------------------------------------------------------------

ATH4:		lde	R13, @RR14
		inc	R15
		cp	R13, #30h
		jr	C, ath4_2
		sub	R13, #30h
		cp	R13, #0Ah
		jr	C, ath4_1
		cp	R13, #11h
		jr	C, ath4_2
		sub	R13, #7
		cp	R13, #10h
ath4_1:		ccf
ath4_2:		ret

;------------------------------------------------------------------------------
; wandelt 4 Ziffern ab (01Eh/1F) nach 01Ch/1D, 012h zerstrt, 
; 01Fh um vier erhht
;------------------------------------------------------------------------------

ATH16:		call	ATH8
		jr	C, ath8_1
		ld	byte_1C, R13

;------------------------------------------------------------------------------
; wandelt 2 Ziffern ab (01Eh/1F) nach 01Dh, 012h zerstrt, 01Fh um zwei erhht
;------------------------------------------------------------------------------

ATH8:		call	ATH4
		jr	C, ath8_1
		ld	R2, R13
		swap	R2
		call	ATH4
		or	R13, R2
ath8_1:		ret

;------------------------------------------------------------------------------
; Q		Rckkehr zum aufrufenden Programm
;------------------------------------------------------------------------------

mon_quit:	pop	R0
		pop	R0
		jp	quit1		; Rcksichern Register 10..1F

;------------------------------------------------------------------------------
; Haaaa		Hexanzeige der Speicherstellen (oder Register, falls 
; 		Adresse 0000-00FF), es werden 4 byte ausgegeben
;------------------------------------------------------------------------------

mon_hexanz:	call	ADRE
hex1:		ld	R5, #','
		call	PCAS		; Ausgabe ',' und Adr.
		ld	R0, #4		; Anz. Bytes
hex2:		lde	R9, @RR10
		or	R10, R10
		jr	NZ, hex3
		ld	R9, @R11
hex3:		incw	RR10
		call	HTA8		; Ausgabe Byte
		djnz	R0, hex2
		call	RWCONT
		jr	Z, hex1
hex4:		ret

;------------------------------------------------------------------------------
; ,aaaa bcbcbcbc	Hexschreiben des Speichers/der Register(0000-00FF), 
;			auch bei weniger als 4 byte werden Daten eingeschrieben, 
;			aber keine neue Adresse ausgegeben
;------------------------------------------------------------------------------

mon_hexwrite:	call	ADRE
		ld	R0, #4
hexw1:		call	ATH8
		jr	C, hex4
		or	R10, R10
		jr	Z, hexw2
		lde	@RR10, R13
		db  0Dh			; nchster Befehl als NOP
hexw2:		ld	@R11, R13
		incw	RR10
		djnz	R0, hexw1
		ld	R5, #2Ch
hexw3:		jp	PCAS

;------------------------------------------------------------------------------
; P	schaltet den Drucker aus
; PO	schaltet den Drucker an
;------------------------------------------------------------------------------

mon_printer:	lde	R5, @RR14
		ld	byte_5D, R5	; Drucker an/aus
		ret
		
;------------------------------------------------------------------------------
; Aaaaa		ASCII -Anzeige des Speichers, die Kodes	00-0F werden als 
;		Leerzeichen angezeigt
;------------------------------------------------------------------------------

mon_ascii:	call	ADRE
ascii1:		ld	R5, #';'
		call	PCAS		; Ausgabe ';' und Adr.
		ld	R0, #8		; 8 Zeichen
ascii2:		lde	R5, @RR10
		tm	R5, #0F0h
		jr	NZ, ascii3
		ld	R5, #20h
ascii3:		incw	RR10
		call	j_CHAROUT
		djnz	R0, ascii2
		call	RWCONT
		jr	Z, ascii1
		ret

;------------------------------------------------------------------------------
; Jaaaa		startet	Maschinenprogramm ab Adresse aaaa mit CALL, das	
;		Programm kann mit RET zurckkehren, das	dabei benutzte Nutzer-
;		Flagregister ist sonst in 016h gespeichert und der Nutzer-
;		Registerpointer in 017h
;------------------------------------------------------------------------------

mon_jump:	call	ADRE
		ld	flags, R6	; Program control flags
		ld	rp, byte_17	; Register pointer
		call	@word_1A	; startet Maschinenprogramm 
		ld	byte_17, rp	; Register pointer
		srp	#10h
		ld	R6, flags	; Program control flags
		ret

		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		
;------------------------------------------------------------------------------
; ;aaaa dddddddd	ASCII-Schreiben, es werden immer 8 byte geschrieben
;------------------------------------------------------------------------------

mon_asciiwrite:	call	ADRE
		ld	R0, #8
asciiw1:	lde	R2, @RR14
		lde	@RR10, R2
		inc	R15
		incw	RR10
		djnz	R0, asciiw1
		ld	R5, #';'
		jr	hexw3

;------------------------------------------------------------------------------
; Saaaa aaaa		speichert ab Adresse aaaa aud Kassette aaaa Bytes
;------------------------------------------------------------------------------

mon_save:	call	ADRE
		ld	word_20, R10
		ld	word_20+1, R11
		call	ADRE
		ld	word_22, R10
		ld	word_22+1, R11
		jp	j_SAVE

;------------------------------------------------------------------------------
; Laaaa			ldt von Kassette in Speicher ab Adresse aaaa,
;			druckt dann aaaa bc aus, aaaa ist geladene byteanzahl 
;			und bc Fehlermeldung (00: fehlerfrei)
;------------------------------------------------------------------------------

mon_load:	call	ADRE
		ld	word_20, R10
		ld	word_20+1, R11
		call	j_LOAD
		ld	R8, word_22
		ld	R9, word_22+1
		call	HTA16
		ld	R5, #' '
		call	j_CHAROUT
		ld	R9, byte_24
		call	HTA8
mload1:		jp	PRRET

;------------------------------------------------------------------------------
; Ende MON
;------------------------------------------------------------------------------

quit1:		srp	#60h		; Rcksichern Register 10..1F
		ld	R0, #0F7h	; Zwischenspeicher
		ld	R1, #80h
		ld	R2, #10h	; erstes Register
		ld	R3, #10h	; Anzahl
quit2:		ldei	@R2, @RR0
		djnz	R3, quit2
		pop	rp		; Register pointer
		ret

		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh

;------------------------------------------------------------------------------
; ?aaaa aaaa		gibt aaaa plus aaaa und	aaaa minus aaaa	aus
;------------------------------------------------------------------------------

mon_arith:	call	ADRE		; RR10 = 1. Wert
		call	ATH16		; RR12 = 2. Wert
		jr	C, arith1
		ld	R8, R10
		ld	R9, R11
		add	R9, R13
		adc	R8, R12
		call	HTA16		; Ausgabe Summe
		ld	R5, #' '
		sub	R11, R13
		sbc	R10, R12
		call	PCAS		; Ausgabe ' ' und Differenz
		jr	mload1
arith1:		ret

;------------------------------------------------------------------------------
; Erweiterungsbefehle
; Daaaa			springt nach %C100, Disassemblieren
;------------------------------------------------------------------------------

mon_disass:	call	ADRE
		jp	0C100h

;------------------------------------------------------------------------------
; .aaaa			springt nach %C103, Assemblieren
;------------------------------------------------------------------------------

mon_assembl:	call	ADRE
		jp	0C103h

;------------------------------------------------------------------------------
; Exaaaa	Aufruf irgendwelcher Funktionen, x ist das ASCII-Zeichen 
;		das als Funktionsname dient.
; Ex	springt nach %C106, der ASCII-Code von x liegt in %13
;------------------------------------------------------------------------------

mon_extra:	lde	R3, @RR14
		inc	R15
		call	ADRE
		jp	0C106h

		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		
;------------------------------------------------------------------------------
; Zeilennummer, FSE
;------------------------------------------------------------------------------

kdo15:		cp	R6, #3Ah	; '9'+1
kdo16:		jp	NC, kdoloop
		cp	R6, #30h	; '0'
		ccf
		jr	NC, kdo16
		ld	R15, #0
		ld	R14, #0F7h
		call	DA5TH16
		ccf
		jr	NC, kdo16
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		or	R4, #80h
		call	sub_EC5
inp1:		lde	R10, @RR0
		incw	RR0
		lde	R11, @RR0
		decw	RR0
		or	R10, R10
		jr	Z, inp9
		cp	R10, R4
		jr	C, inp2
		jr	UGT, inp9
		cp	R11, R5
		jr	Z, inp4
		jr	UGT, inp9
inp2:		incw	RR0
inp3:		incw	RR0
		lde	R10, @RR0
		cp	R10, #0Dh
		jr	NZ, inp3
		incw	RR0
		jr	inp1
inp4:		ld	R6, R0
		ld	R7, R1
		incw	RR6
inp5:		incw	RR6
		lde	R10, @RR6
		cp	R10, #0Dh
		jr	NZ, inp5
		incw	RR6
		ld	R8, R0
		ld	R9, R1
inp6:		lde	R10, @RR6
		lde	@RR8, R10
		cp	R6, R2
		jr	NZ, inp7
		cp	R7, R3
		jr	Z, inp8
inp7:		incw	RR6
		incw	RR8
		jr	inp6
inp8:		ld	R2, R8
		ld	R3, R9
inp9:		ld	R15, #4
		ld	R14, byte_58	; Zeilenlnge
		cp	R14, R15
		jp	ULE, kdoloop
		ld	R7, #0FFh
		add	R7, R14
		ld	R6, #0
		add	R7, R3
		adc	R6, R2
inp10:		lde	R10, @RR2
		lde	@RR6, R10
		cp	R0, R2
		jr	NZ, inp11
		cp	R1, R3
		jr	Z, inp12
inp11:		decw	RR2
		decw	RR6
		jr	inp10
inp12:		lde	@RR0, R4
		incw	RR0
		lde	@RR0, R5
		incw	RR0
		ld	R2, #0F7h
		ld	R3, #5
		sub	R14, R15
inp13:		lde	R10, @RR2
		lde	@RR0, R10
		incw	RR0
		inc	R3
		dec	byte_1E
		jr	NZ, inp13
		ld	R10, #0Dh
		lde	@RR0, R10
		jp	kdoloop

sub_EC5:	ld	R2, R0
		ld	R3, R1
loc_EC9:	lde	R10, @RR2
		tm	R10, #80h
		jr	Z, locret_EDF
		incw	RR2
loc_ED2:	incw	RR2
		lde	R10, @RR2
		cp	R10, #0Dh
		jr	NZ, loc_ED2
		incw	RR2
		jr	loc_EC9
locret_EDF:	ret

;------------------------------------------------------------------------------
; Dezimalzahl ausgeben
;------------------------------------------------------------------------------

PrDez:		call	182h		; U883BAS: todez, Word 012h-13 in ASCII 015h-19 +	Vorzeichen in R4
		cp	R4, #20h
		jr	Z, prDez1
		push	byte_15
		ld	R5, R4
		call	j_CHAROUT
		pop	byte_15
prDez1:		ld	R11, #15h
prDez2:		ld	R5, @R11
		call	j_CHAROUT
		inc	R11
		cp	R11, #1Ah
		jr	C, prDez2
		ret

		db 0FFh

;------------------------------------------------------------------------------
; Tastaturtabelle
;------------------------------------------------------------------------------

		org	0F00h
		
		db 0FFh, 'Y', 'X', 'C', 'V', 'B', 'N', 'M', ',', '.', '/', 0Dh, ' ', ' ', 4, 0Dh
		db 0FFh, 'A', 'S', 'D', 'F', 'G', 'H', 'J', 'K', 'L', ';', '+', '*', 1, 5, 2
		db 0FFh, 'Q', 'W', 'E', 'R', 'T', 'Z', 'U', 'I', 'O', 'P', '-', '=', 7, 3, 6
		db 0FFh, '1', '2', '3', '4', '5', '6', '7', '8', '9', '0', '<', '>', 8, 9, 0Ch

; 1	Kursor links
; 2	Kursor rechts
; 3	Kursor hoch
; 4	Kursor runter
; 5	HOME. Kursor auf Bildanfang
; 6	SOL, Kursor auf Zeilenanfang
; 7	DEL, Zeichen herauslschen
; 8	DBS, lschender Rckschritt
; 9	INS, Leerzeichen einfgen
; 0C	CLS, Bildschirm lschen und Kursor auf Bildanfang
; 0D	RETURN

;------------------------------------------------------------------------------
; Tastaturtabelle Shift
;------------------------------------------------------------------------------

		db 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, '[', ']', '?', 0FFh, 0FFh, 0FFh, 0FFh, 0FFh
		db 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, ':', '\\', '^', 0FFh, 0FFh, 0FFh
		db 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, '_', 0FFh, 0FFh, 0FFh
		db 0FFh, 0FFh, '!', '"', '#', '$', '%', '&', 27h, '@', '(', ')', 0FFh, 0FFh, 0FFh, 0FFh, 0FFh

;------------------------------------------------------------------------------
; Maaaa aaaa aaaa	verschiebt von Adresse aaaa nach aaaa aaaa bytes 
;			im Speicher. auch bei berschneidung des Quell- und 
;			Zielbereichs erfolgt immer korrekte Verschiebung
;------------------------------------------------------------------------------

mon_move:	call	ADRE
		ld	R8, R10
		ld	R9, R11
		call	ADRE
		call	ATH16
		jr	C, move3
		cp	R8, R10
		jr	UGT, move4
		jr	C, move1
		cp	R9, R11
		jr	NC, move4
move1:		add	R9, R13
		adc	R8, R12
		add	R11, R13
		adc	R10, R12
move2:		decw	RR8
		decw	RR10
		lde	R0, @RR8
		lde	@RR10, R0
		decw	RR12
		jr	NZ, move2
move3:		ret
		;
move4:		lde	R0, @RR8
		lde	@RR10, R0
		incw	RR8
		incw	RR10
		decw	RR12
		jr	NZ, move4
		ret
		
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh

;------------------------------------------------------------------------------
; Zeichensatztabelle 8x8
; Bitmuster hier nur fr die ASCII-Kodes %20-5F. 
; Die Startadresse ist daher %0F00 (s. Initialisierung in KOMMAND)
;------------------------------------------------------------------------------

		org 1000h
		include	es23_0800_zg.asm
		
;------------------------------------------------------------------------------
; CHAROUT	fhrt das Zeichen aus 015h auf dem Bildschirm aus und gibt es 
;		evtl. noch ber 0F512h aus
;------------------------------------------------------------------------------

CHAROUT:	cp	byte_5D, #'O'	; Drucker an/aus
		jr	NZ, co1
		call	0F512h
;
co1:		ld	byte_5A, byte_15
co2:		push	rp		; Register pointer
		srp	#50h
		ld	R12, byte_55	; log. Zeilenlnge
		dec	byte_5C
		tm	byte_5A, #0F0h
		jr	Z, ctrl01	; wenn Control-Code 0x
		call	sub_8E4		; Zeichen in ASCII-Speicher und BWS schreiben
		ld	R12, #1
co3:		add	R11, R12
		jr	C, co4
		pop	rp		; Register pointer
		ret
; Zeilenvorschub
co4:		sub	R11, R5
		srp	#60h
		ld	R0, #0F4h	; F400 ASCII-Textspeicher
		ld	R2, #0F4h
		ld	R1, byte_55	; log. Zeilenlnge
		ld	R3, #0
co5:		lde	R4, @RR0
		lde	@RR2, R4
		inc	R3
		inc	R1
		jr	NZ, co5
		ld	R4, #20h
co6:		lde	@RR2, R4
		inc	R3
		jr	NZ, co6
		ld	R2, #0F8h	; F800 Bildwiederholspeicher
		ld	R0, #1Fh
		ld	R1, byte_55	; log. Zeilenlnge
		ld	R4, #3
		rcf
co7:		rlc	R1
		rlc	R0
		djnz	R4, co7
co8:		lde	R4, @RR0
		lde	@RR2, R4
		incw	RR2
		incw	RR0
		jr	NZ, co8
		ld	R4, #0FFh
co9:		lde	@RR2, R4
		incw	RR2
		jr	NZ, co9
co10:		pop	rp		; Register pointer
		ret

;------------------------------------------------------------------------------
; Control-Codes, Steuerkodes
; 00h
; 01h	Kursor links
; 02h	Kursor rechts
; 03h	Kursor hoch
; 04h	Kursor runter
; 05h	HOME. Kursor auf Bildanfang
; 06h	SOL, Kursor auf	Zeilenanfang
; 07h	DEL, Zeichen herauslschen
; 08h	DBS, lschender	Rckschritt
; 09h	INS, Leerzeichen einfgen
; 0Ah
; 0Bh
; 0Ch	CLS, Bildschirm	lschen	und Kursor auf Bildanfang
; 0Dh	RETURN
; 0Eh
; 0Fh
;------------------------------------------------------------------------------

; 01h	Kursor links
ctrl01:		djnz	R10, ctrl02
		dec	cupos		; Kursorposition
		jr	co10

; 02h	Kursor rechts
ctrl02:		djnz	R10, ctrl03
		inc	R11		; Pos+1
		jr	co10

; 03h	Kursor hoch
ctrl03:		djnz	R10, ctrl04
		sub	R11, R5
		jr	co10

; 04h	Kursor runter
ctrl04:		djnz	R10, ctrl05
loc_1276:	ld	R12, byte_55	; log. Zeilenlnge
		jr	co3

; 05h	HOME, Kursor auf Bildanfang
ctrl05:		djnz	R10, ctrl06
		ld	R11, #0		; Pos. 0
		jr	co10

; 06h	SOL, Kursor auf	Zeilenanfang
ctrl06:		djnz	R10, ctrl07
		com	byte_5C
		and	R11, R12
		jr	co10

; 07h	DEL, Zeichen herauslschen
ctrl07:		djnz	R10, ctrl08
		push	cupos		; Kursorposition
		ld	R14, #0F4h
		ld	R15, cupos	; Kursorposition
loc_1290:	tcm	R15, R12
		jr	Z, loc_129D
		inc	R15
		lde	R10, @RR14	; R10==byte_5A
		call	sub_8E4		; Zeichen in ASCII-Speicher und BWS schreiben
		inc	R11
		jr	loc_1290
loc_129D:	ld	R10, #' '	; R10==byte_5A
		call	sub_8E4		; Zeichen in ASCII-Speicher und BWS schreiben
		pop	cupos		; Kursorposition
loc_12A4:	pop	rp		; Register pointer
		ret

; 08h	DBS, lschender	Rckschritt
ctrl08:		djnz	R10, ctrl09
		tm	R11, R12
		jr	Z, loc_12B4
		dec	cupos		; Kursorposition
		ld	R10, #' '	; R10==byte_5A
		call	sub_8E4		; Zeichen in ASCII-Speicher und BWS schreiben
loc_12B4:	jr	loc_12A4

; 09h	INS, Leerzeichen einfgen
ctrl09:		djnz	R10, ctrl0a
		tcm	R11, R12
		jr	Z, loc_12D3
		ld	R9, cupos	; Kursorposition
		ld	R14, #0F4h
		ld	R15, cupos	; Kursorposition
		or	R15, R12
		ld	R11, byte_5F
loc_12C6:	dec	byte_5F
		lde	R10, @RR14
		call	sub_8E4		; Zeichen in ASCII-Speicher und BWS schreiben
		dec	cupos		; Kursorposition
		cp	R11, R9
		jr	NZ, loc_12C6
loc_12D3:	ld	R10, #' '	; R10==byte_5A
		call	sub_8E4		; Zeichen in ASCII-Speicher und BWS schreiben
		jr	loc_12A4

; 0Ah -- (cls)
ctrl0a:		djnz	R10, ctrl0b
		nop
		nop

; 0Bh -- (cls)
ctrl0b:		djnz	R10, ctrl0c
		nop
		nop

; 0Ch	CLS, Bildschirm	lschen	und Kursor auf Bildanfang
ctrl0c:		djnz	R10, ctrl0d
		ld	R11, #0		; 256 (16x16)
loc_12E6:	ld	R10, #' '	; R10==byte_5A
		call	sub_8E4		; Zeichen in ASCII-Speicher und BWS schreiben
		inc	R11
		jr	NZ, loc_12E6
		jr	loc_12A4

; 0Dh	RETURN
ctrl0d:		djnz	R10, ctrl0e
		com	byte_5C
		and	R11, R12
		jp	loc_1276

; 0Eh, 0Fh --
ctrl0e:		pop	rp		; Register pointer
		ret

		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh

;------------------------------------------------------------------------------
; CHARIN	liefert	in 013h nacheinander die Zeichen einer vom FSE 
; editierten Zeile, als letztes den Kode 00Dh. Ist 058h auf 0FFh gesetzt, dann
; wird der FSE aufgerufen, auch wenn die alte Zeile noch nicht vollstndig 
; bergeben wurde, und sofort eine neue bearbeitet. Bei der ersten Rckkehr 
; nach Zeilenbearbeitung steht in 058h die Byteanzahl-1	der Zeile und die Zeile	
; liegt ab 0F700h im Speicher, bis zur max. logischen Zeilenlnge durch 
; Leerzeichen ergnzt oder 00Dh-Kode.
;------------------------------------------------------------------------------

CHARIN:		push	rp		; Register pointer
		srp	#50h
		or	R8, R8
		jr	MI, ci4
ci1:		dec	byte_58		; Zeilenlnge
		jr	PL, ci3
		ld	R12, #0Dh
ci2:		ld	byte_13, R12
		pop	rp		; Register pointer
		ret
ci3:		ld	R14, #0F7h
		ld	R15, byte_66
		lde	R12, @RR14
		inc	byte_66
		jr	ci2
ci4:		call	j_WKEY
		ld	R10, byte_13
		call	co2
		cp	byte_13, #0Dh
		jr	NZ, ci4
		ld	R8, #0F4h
		ld	R9, cupos	; Kursorposition
		sub	R9, R5
		ld	R14, #0F7h
		ld	R15, #0
		ld	R10, #0
		nop
		nop
		nop
		nop
ci5:		lde	R12, @RR8
		lde	@RR14, R12
		cp	byte_5C, #' '
		jr	Z, ci6
		nop
		nop
		nop
		nop
		ld	R10, byte_5F
		inc	R10
		nop
		nop
		nop
		nop
ci6:		inc	R15
		inc	R9
		cp	R9, R11
		jr	NZ, ci5
		nop
		nop
		ld	R8, byte_5A
		ld	byte_66, #0
		jr	ci1

		db 0FFh
		db 0FFh
		db 0FFh
		
;------------------------------------------------------------------------------
; WKEY	wartet mit Kursordarstellung auf Tastendruck, liefert Ascii-Kode in 013h
;------------------------------------------------------------------------------

WKEY:		push	rp		; Register pointer
		srp	#60h
		call	sub_8CD		; ASCII Cursorpos -> graph. BWS-Adr 
		add	R5, #70h
		lde	R0, @RR4
		push	R0
		com	R0
		lde	@RR4, R0
		ld	byte_5F, R13
		db  0Dh			; next code as nop
wkey1:		ld	R12, #0
		call	j_KEY
		ld	byte_13, R13
		or	R13, R13
		jr	Z, wkey1
		ld	R0, #6
		add	R0, R15
wkey2:		cp	R0, R15
		jr	NZ, wkey2
		or	R12, R12
		jr	Z, wkey4
		cp	byte_6D, byte_5F ; gedr. Taste
		jr	Z, wkey7
		ld	R0, #5
		add	R0, R15
wkey3:		cp	R0, R15
		jr	NZ, wkey3
wkey4:		ld	byte_5F, #14h
wkey5:		ld	byte_5E, R15
		call	j_KEY
		or	R13, R13
		jr	Z, wkey7
wkey6:		cp	byte_5E, byte_6F
		jr	Z, wkey6
		dec	byte_5F
		jr	NZ, wkey5
		ld	R12, #0FFh
wkey7:		pop	R0
		lde	@RR4, R0
		pop	rp		; Register pointer
		ret

		db 0FFh

;------------------------------------------------------------------------------
; KEY	liefert	den Ascii-Kode der momentan gedrckten Taste in	06Dh, 
; wenn keine Taste dann 00h
;------------------------------------------------------------------------------

KEY:		push	rp		; Register pointer
		srp	#60h
		ld	R0, #7Fh
		ld	R1, #0Fh
key1:		ldc	R2, @RR0
		and	R2, #0F0h
		jr	NZ, key3
		djnz	R1, key1
		ld	R13, #0
key2:		pop	rp		; Register pointer
		ret
key3:		ld	R3, #40h
key4:		sub	R3, #10h
		rl	R2
		jr	NC, key4
		or	R1, R3
		ld	R2, #7Fh	; Zeilenspeicher
		ld	R3, #0
		ldc	R0, @RR2
		ld	R2, #0F7h	; CONTROL-Belegung der Tasten
		ld	R3, #40h
		add	R3, R1
		rl	byte_60
		rl	byte_60
		rl	byte_60
		jr	C, key5
		ld	R2, #0Fh
		rl	byte_60
		jr	C, key5
		sub	byte_63, #40h
key5:		ldc	R13, @RR2
		jr	key2

		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh

;------------------------------------------------------------------------------
; ; Sichern der Register 04..0F
;------------------------------------------------------------------------------

savreg00:	push	rp		; Register pointer
		srp	#60h
		ld	R0, #0F7h
		ld	R1, #90h
		ld	R2, #4
		ld	R3, #0Ch
savreg00_1:	ldei	@RR0, @R2
		djnz	R3, savreg00_1
		pop	rp		; Register pointer
		ret
		
		db 0FFh

;------------------------------------------------------------------------------
; Restaurieren Register 04..0F
;------------------------------------------------------------------------------

rstreg00:	push	rp		; Register pointer
		srp	#60h
		ld	R0, #0F7h
		ld	R1, #90h
		ld	R2, #4
		ld	R3, #0Ch
rstreg00_1:	ldei	@R2, @RR0
		djnz	R3, rstreg00_1
		pop	rp		; Register pointer
		ret

		db 0FFh

;------------------------------------------------------------------------------
; init. Systemregister
;------------------------------------------------------------------------------

sub_1428:	di
		ld	byte_5E, #0FFh
		ld	p01m, #92h	; Ports	0-1 mode
		ld	t0, #64h	; Timer/counter	0
		ld	pre0, #5	; T0 prescaler
		ld	tmr, #43h	; Timer	mode
		ld	p3m, #8		; Port 3 mode
		ret
		
		db 0FFh

;------------------------------------------------------------------------------
; 
;------------------------------------------------------------------------------

sub_143D:	call	sub_1428
		ld	byte_D,	#0
		ld	tmr, #3		; Timer	mode
		ret

		db 0FFh

;------------------------------------------------------------------------------
; 
;------------------------------------------------------------------------------

sub_1448:	call	sub_1428
		push	rp		; Register pointer
		srp	#0
		ld	R4, #0
		ld	R5, #0
loc_1453:	ld	R6, #18h
loc_1455:	djnz	R6, loc_1455
		decw	word_4
		jr	NZ, loc_1453
		ld	R13, #80h	; F580-F5FF	  Datenbytes
		pop	rp		; Register pointer
		ret

;------------------------------------------------------------------------------
; 
;------------------------------------------------------------------------------

sub_1460:	push	rp		; Register pointer
		srp	#0
		or	R13, R13
		jr	NZ, loc_146F
		call	ld_block
		jr	C, loc_1479
		ld	R13, #80h	; F580-F5FF	  Datenbytes
loc_146F:	lde	R15, @RR12	; nchstes Byte
		inc	R13
		cp	R13, R11	; Ende?
		jr	NZ, loc_1478
		ld	R13, #0
loc_1478:	rcf
loc_1479:	pop	rp		; Register pointer
		ret

		db 0FFh
		db 0FFh

;------------------------------------------------------------------------------
; 
;------------------------------------------------------------------------------

sub_147E:	push	rp		; Register pointer
		srp	#0
		ld	R12, #0F5h
		lde	@RR12, R15
		inc	R13
		jr	NZ, loc_1490
		ld	R11, #0FCh	; FC	  128 Datenbyte
		call	wr_block
		ld	R13, #80h
loc_1490:	pop	rp		; Register pointer
		ret

		db 0FFh

;------------------------------------------------------------------------------
; save last block + close
;------------------------------------------------------------------------------

sub_1494:	cp	byte_D,	#80h	; Anzahl Bytes
		jr	Z, loc_149F
		ld	byte_B,	#0FAh	; FC	  128 Datenbyte
		call	wr_block
loc_149F:	ld	byte_B,	#0FEh	; End of File Block
		call	wr_block
sub_14A5:	call	sub_910		; set system registers
		ret

		db 0FFh

;------------------------------------------------------------------------------
; 
;------------------------------------------------------------------------------

wait:		ld	byte_60, #18h
wait1:		dec	byte_60
		jr	NZ, wait1
		ret

		db 0FFh
		db 0FFh

;------------------------------------------------------------------------------
; LOAD	Speicherbereich	von Kassette laden, in 020h/21 Anfangsadresse 
; bergeben, Rckgabewerte: 022h/23 Anzahl der geladenen Byte, 
; 24h Status (00 fehlerfrei, 0FFh Ladefehler)
;------------------------------------------------------------------------------

LOAD:		call	savreg00	; Sichern der Register 04..0F
		call	sub_143D
		push	rp		; Register pointer
		srp	#20h
		ld	R2, #0
		ld	R3, #0
		ld	R4, #0
load1:		call	sub_1460
		jr	C, load2
		ld	R5, byte_F
		lde	@RR0, R5
		incw	word_20
		incw	word_22
		jr	load1
load2:		cp	byte_E,	#88h
		jr	Z, load3
		ld	R4, byte_E
load3:		call	sub_14A5
load4:		call	rstreg00	; Restaurieren Register	04..0F
		pop	rp		; Register pointer
		ret

		db 0FFh

;------------------------------------------------------------------------------
; SAVE	Speicherbereich	auf Kassette speichern,	in 020h/21 Anfangsadresse, 
; in 022h/23 Lnge bergeben
;------------------------------------------------------------------------------

SAVE:		call	savreg00	; Sichern der Register 04..0F
		call	sub_1448
		push	rp		; Register pointer
		srp	#20h
save1:		lde	R5, @RR0
		ld	byte_F,	R5
		call	sub_147E
		incw	word_20
		decw	word_22
		jr	NZ, save1
		call	sub_1494
		jr	load4

;------------------------------------------------------------------------------
; Block schreiben
;------------------------------------------------------------------------------

wr_block:	push	rp		; Register pointer
		srp	#0
		ld	R4, #90h
		ld	R5, #0
wr_block1:	decw	word_4
		jr	NZ, wr_block1
		ld	R4, #0F5h	; F5FF	  Ende Kassettenpuffer
		ld	R5, #0FFh
		cp	byte_B,	#0FAh	; Formatbyte
		jr	NZ, wr_block2
		and	byte_D,	#7Fh
		lde	@RR4, R13	; bei FA Anzahl	Bytes auf 0F5FFh
wr_block2:	ld	R5, #7Fh	; F57F		  Formatbyte
		lde	@RR4, R11
		ld	R6, #0
		rcf
wr_block3:	lde	R7, @RR4
		adc	R6, R7
		inc	R5
		jr	NZ, wr_block3
		adc	byte_6,	#0
		ld	R5, #7Bh	; F57B		  Blocknummer
		inc	byte_5E
		ld	R7, byte_5E
		lde	@RR4, R7
		inc	R5
		lde	@RR4, R7
		inc	R5
		lde	@RR4, R6	; F57D		  Prfsumme
		inc	R5
		lde	@RR4, R6
		ld	R5, #7Bh
		ld	tmr, #3		; Timer	mode
		ld	R3, #40h
		call	wait
		ld	R10, #40h
		ld	R3, #0
		ld	R6, #4
wr_block4:	call	wait
		djnz	R6, wr_block4
wr_block5:	lde	R6, @RR4
		ld	R7, #8
wr_block6:	rl	byte_6
		xor	R3, R10
		call	wait
		jr	NC, wr_block7
		call	wait
wr_block7:	xor	R3, R10
		call	wait
		jr	NC, wr_block8
		call	wait
wr_block8:	djnz	R7, wr_block6
		inc	R5
		jr	NZ, wr_block5
		xor	R3, R10
		call	wait
		ld	tmr, #43h	; Timer	mode
		pop	rp		; Register pointer
		ret

		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh

;------------------------------------------------------------------------------
; Block lesen
;------------------------------------------------------------------------------

ld_block:	push	rp		; Register pointer
		srp	#0
ld_block1:	ld	R4, #0F5h	; F57B		  Kassettenpuffer
		ld	R5, #7Bh
		ld	R10, #3Dh
		ld	R9, #2Ch
		ld	R6, p3		; Port 3
ld_block2:	cp	R3, R6
		jr	Z, ld_block2
ld_block3:	ld	R6, p3		; Port 3
		ld	R7, #58h
ld_block4:	dec	byte_7
		jr	Z, ld_block3
		cp	R3, R6
		jr	Z, ld_block4
		cp	R7, R9
		jr	NC, ld_block3
ld_block5:	ld	R8, #8
ld_block6:	ld	R7, #0
		ld	R6, p3		; Port 3
ld_block7:	inc	R7
		cp	R3, R6
		jr	Z, ld_block7
		ld	R6, p3		; Port 3
ld_block8:	inc	R7
		cp	R3, R6
		jr	Z, ld_block8
		cp	R10, R7
		rlc	byte_9
		djnz	R8, ld_block6
		lde	@RR4, R9	; Byte in Kassettenpuffer
		inc	R5
		jr	NZ, ld_block5	; weiter bis Pufferende
		ld	R5, #7Dh	; F57D		  Prfsumme
		ld	R7, #0
		lde	R10, @RR4
		inc	R5
		lde	R9, @RR4
		inc	R5
		rcf
ld_block9:	lde	R6, @RR4
		adc	R7, R6		; Summieren
		inc	R5
		jr	NZ, ld_block9
		adc	byte_7,	#0
		cp	R7, R9		; Vergleich mit	Prfsumme
		jr	NZ, ld_block13
		cp	R7, R10
		jr	NZ, ld_block13
		ld	R5, #7Bh	; F57B		  Blocknummer
		lde	R9, @RR4
		inc	R5
		lde	R10, @RR4
		ld	R6, byte_5E
		inc	R6
		cp	R6, R9
		jr	NZ, ld_block13
		cp	R6, R10
		jr	NZ, ld_block13
		ld	R5, #7Fh	; F57F		  Formatbyte
		lde	R7, @RR4
		ld	R11, #0
		cp	byte_7,	#0FCh	; FC	  128 Datenbyte
		jr	Z, ld_block12
		ld	R5, #0FFh
		lde	R8, @RR4
		ld	R11, #80h
		or	R11, R8
		cp	byte_7,	#0FAh	; FA	  weniger als 128 Datenbyte
		jr	Z, ld_block12
		cp	byte_7,	#0FEh	; FE	  End of File Block
		jr	NZ, ld_block13
		ld	R14, #88h
ld_block10:	scf
ld_block11:	ld	R12, #0F5h
		pop	rp		; Register pointer
		ret
ld_block12:	ld	byte_5E, R6
		ld	tmr, #3		; Timer	mode
		rcf
		jr	ld_block11
ld_block13:	ld	tmr, #43h	; Timer	mode
ld_block14:	call	j_KEY
		cp	byte_6D, #'B'	; gedr.	Taste, Abbruch
		jr	Z, ld_block16
		cp	byte_6D, #'C'	; gedr.	Taste, Continue
		jr	NZ, ld_block14
		ld	R4, #7
ld_block15:	djnz	R5, ld_block15
		djnz	R4, ld_block15
		ld	tmr, #3		; Timer	mode
		jp	ld_block1
ld_block16:	ld	R14, #0FFh
		jr	ld_block10

		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh

;------------------------------------------------------------------------------
; Faaaa aaaa bc	fllt Speicher ab Adresse aaaa mit aaaa	bytes bc
;------------------------------------------------------------------------------

mon_fill:	call	ADRE
		ld	R8, R10
		ld	R9, R11
		call	ADRE
		call	ATH8
		jr	C, fill2
fill1:		lde	@RR8, R13
		incw	RR8
		decw	RR10
		jr	NZ, fill1
fill2:		ret
		
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh

;------------------------------------------------------------------------------
; Grafikroutinen
;------------------------------------------------------------------------------

; Berechne BWS-Adr
ppos:		ld	R0, var_Y_lo
		ld	R1, var_X_lo
		ld	R3, #7
		and	R3, R1
		rl	R1
		swap	R1
		and	R1, #0Fh
		swap	R0
		ld	R2, #0F0h
		and	R2, R0
		or	R1, R2
		or	R0, #0F8h	; F800	Bildwiederholspeicher
		inc	R3
		ld	R2, #0
		scf
ppos1:		rrc	R2
		djnz	R3, ppos1
		ret

		db 0FFh

;------------------------------------------------------------------------------
; PLOT	setzt auf Position aus 04Fh (x) und 051h (y) einen Punkt.
; benutzt Register 60h-63
;------------------------------------------------------------------------------

PLOT:		push	rp		; Register pointer
		srp	#60h
		call	ppos		; Berechne Adr
		lde	R3, @RR0
		tm	var_Z_lo, #1
		jr	Z, plot1
		or	R3, R2
		jr	plot2
plot1:		com	R2
		and	R3, R2
plot2:		lde	@RR0, R3
		pop	rp		; Register pointer
		ret

		db 0FFh

;------------------------------------------------------------------------------
; PTEST	liefert	in 053h die Farbe des Punktes auf Position aus 04Fh (x) 
; und 051h (y). benutzt Register 60h-63
;------------------------------------------------------------------------------

PTEST:		push	rp		; Register pointer
		srp	#60h
		call	ppos		; Berechne Adr
		ld	var_Z_lo, #0
		lde	R3, @RR0
		tm	R3, R2
		jr	Z, ptest1
		ld	var_Z_lo, #1
ptest1:		pop	rp		; Register pointer
		ret

		db 0FFh
		db 0FFh

;------------------------------------------------------------------------------
; 
;------------------------------------------------------------------------------

p_draw1:	ld	var_Y_lo, R7
		ld	var_X_lo, R6
		jp	PLOT
		
		db 0FFh

p_draw2:	ld	var_Y_lo, R6
		ld	var_X_lo, R7
		jp	PLOT

		db 0FFh

;------------------------------------------------------------------------------
; DRAW	verbindet die Punkte, deren Koordinaten	in 04Bh (x) und 04Dh (y) 
; sowie 04Fh (x) und 051h bergeben wurden, durch eine Linie. 
; benutzt Register 60h-63 und 72h-7F
;------------------------------------------------------------------------------

DRAW:		push	rp		; Register pointer
		srp	#70h
		ld	R4, var_X_lo
		ld	R5, var_Y_lo
		ld	R6, var_V_lo
		ld	R7, var_W_lo
		ld	R8, #0
		ld	word_72, #hi(p_draw1)
		ld	word_72+1, #lo(p_draw1)
		ld	R11, R6
		sub	R11, R4
		jr	PL, draw1
		com	R11
		inc	R11
draw1:		ld	R12, R7
		sub	R12, R5
		jr	PL, draw2
		com	R12
		inc	R12
draw2:		cp	R11, R12
		jr	NC, draw3
		ld	word_72, #hi(p_draw2)
		ld	word_72+1, #lo(p_draw2)
		ld	R11, R4
		ld	R4, R5
		ld	R5, R11
		ld	R11, R6
		ld	R6, R7
		ld	R7, R11
draw3:		cp	R6, R4
		jr	C, draw4
		ld	R11, R4
		ld	R4, R6
		ld	R6, R11
		ld	R11, R5
		ld	R5, R7
		ld	R7, R11
draw4:		ld	R15, #0
		ld	R9, #0
		ld	R10, #0
		ld	R11, R5
		sub	R11, R7
		jr	Z, draw8
		jr	PL, draw5
		ld	R15, #80h
		com	R11
		inc	R11
draw5:		ld	R14, R4
		sub	R14, R6
		ld	R12, #0
		cp	R11, R14
		jr	NZ, draw6
		inc	R9
		jr	draw7
draw6:		sub	R12, R14
		sbc	R11, #0
		jr	MI, draw7
		inc	R10
		jr	draw6
draw7:		or	R15, R15
		jr	PL, draw8
		com	R10
		com	R9
		inc	R10
		jr	NZ, draw8
		inc	R9
draw8:		call	@word_72
		cp	R6, R4
		jr	NC, draw9
		inc	R6
		add	R8, R10
		adc	R7, R9
		jr	draw8
draw9:		pop	rp		; Register pointer
		ret

		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh

;------------------------------------------------------------------------------
; Druckerausgabe auf Port 2, Centronic-Interface
; Aktivieren mit MON-Befehl ",F512 8D1760"
;------------------------------------------------------------------------------

centr:		ld	p2, #30h	; Port 2
		ld	p2m, #0Fh	; Port 2 mode
		nop
		nop
		push	rp		; Register pointer
		srp	#60h
		ld	R0, #8
		ld	R1, byte_15
centr1:		ld	R2, #0
		rrc	byte_61
		rrc	byte_62
		or	byte_62, #30h
		ld	R4, byte_62
		ld	p2, R2		; Port 2
		ld	R3, #18h
centr2:		djnz	R3, centr2
		or	byte_62, #40h
		ld	p2, R2		; Port 2
		ld	R3, #18h
centr3:		djnz	R3, centr3
		ld	p2, R4		; Port 2
		ld	R3, #18h
centr4:		djnz	R3, centr4
		djnz	R0, centr1
centr5:		ld	R0, #8
		and	byte_60, p3	; Port 3
		jr	NZ, centr5
		ld	p2, #10h	; Port 2
		ld	R3, #18h
centr6:		djnz	R3, centr6
		ld	p2, R4		; Port 2
		pop	rp		; Register pointer
		ret

;------------------------------------------------------------------------------
; SCRFUN	Spezialfunktionen fr den Zugriff auf den Textbildschirm. 
; Momentan existieren 3 Funktionen, die durch den in 053h an SCRFUN bergebenen 
; Wert unterschieden werden.
; Wert:	00h 	liefert	in 04Fh (X, Spalte) und 051h (Y, Zeile) die aktuelle Position 
; 		des Textkursors.
; 	01h	setzt den Textkursor auf die in	04Fh (X, Spalte) und 051h (Y, Zeile) 
; 		bergebene Position. Werte von 0 bis 15 sind sinnvoll und 
; 		erreichbar, denn es werden nur die bits 3 2 1 0 verwendet.
; 	02h	liefert	in 053h Z den ASCII-Code des Zeichens auf der in 04Fh 
; 		(X, Spalte) und 051h (Y, Zeile) bergebenen Position.
; Nach Aufruf von	SCRFUN mit 001h oder 002h in 053h enthalten 04Fh und 051h 
; nicht mehr die alten Werte. 053h wird immer verndert.
;------------------------------------------------------------------------------

SCRFUN:		push	rp		; Register pointer
		srp	#50h
		or	R3, R3		; 53h
		jr	NZ, srcf2
; Funktion 00
		ld	R1, #0F0h
		and	R1, R11
		swap	var_Y_lo
		ld	var_X_lo, #0Fh
		and	var_X_lo, cupos ; Kursorposition
srcf1:		pop	rp		; Register pointer
		ret
srcf2:		djnz	R3, srcf3

; Funktion 01
		ld	R11, #0Fh
		and	cupos, var_X_lo ; Kursorposition
		and	var_Y_lo, #0Fh
		swap	var_Y_lo
		or	R11, R1
		jr	srcf1

; Funktion 02
srcf3:		ld	R14, #0F4h
		ld	R15, #0Fh
		and	byte_5F, var_X_lo
		and	var_Y_lo, #0Fh
		swap	var_Y_lo
		or	R15, R1
		lde	R3, @RR14
		jr	srcf1

;------------------------------------------------------------------------------
; BWKEY	wartet mit Kursordarstellung auf Tastendruck, liefert den Ascii-Code 
; in 05Ah (also fast wie WKEY)
;------------------------------------------------------------------------------

BWKEY:		push	byte_13
		call	j_WKEY
		ld	byte_5A, byte_13
		pop	byte_13
		ret

		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh

;------------------------------------------------------------------------------
; Graphiksprungtabelle
;------------------------------------------------------------------------------

		jp	BWKEY
		jp	DRAW
		jp	PTEST
		jp	PLOT

		end
