; es40_0800.bin
; JU+TE TINY ES4.0 Harun Scheutzow 1990
; reass. V. Pohlers 01.2022

		cpu	z8601
		assume RP:0C0h		; prevent register optimization thru AS
		
; AS-Funktionen
hi	function x,(x>>8)&255
lo	function x, x&255

;------------------------------------------------------------------------------
; Register
;------------------------------------------------------------------------------

word_4 equ 4
byte_6 equ 6
byte_7 equ 7
word_8 equ 8
byte_B equ 0Bh
byte_D equ 0Dh
byte_E equ 0Eh
byte_F equ 0Fh
unk_10 equ 10h
byte_12	equ 12h
byte_13	equ 13h
byte_14	equ 14h
byte_15	equ 15h
byte_16	equ 16h
byte_17	equ 17h
byte_18	equ 18h
byte_19	equ 19h
unk_1A equ 1Ah
unk_1B equ 1Bh
word_1C	equ 1Ch
byte_1E	equ 1Eh

byte_20	equ 20h
byte_21	equ 21h
unk_22 equ 22h
unk_23 equ 23h
byte_24	equ 24h
var_V_hi equ 4Ah
var_V_lo equ 4Bh
var_W_hi equ 4Ch
var_W_lo equ 4Dh
var_X_hi equ 4Eh
var_X_lo equ 4Fh
var_Y_lo equ 51h
var_Z_hi equ 52h
var_Z_lo equ 53h

byte_55	equ 55h
word_56	equ 56h
byte_58	equ 58h
unk_5A equ 5Ah
cupos equ 5Bh
byte_5C	equ 5Ch
byte_5E	equ 5Eh
byte_5F	equ 5Fh
byte_60	equ 60h
unk_61 equ 61h
unk_62 equ 62h
unk_63 equ 63h
unk_64 equ 64h
unk_65 equ 65h
byte_6C	equ 6Ch
byte_6D	equ 6Dh
byte_6F	equ 6Fh
unk_70 equ 70h
unk_71 equ 71h
byte_74	equ 74h
byte_75	equ 75h
byte_76	equ 76h
byte_77	equ 77h
byte_78	equ 78h

;------------------------------------------------------------------------------
; Speicher 
;------------------------------------------------------------------------------

; Register %67		Zeichensatz (hi-Byte)
; 
; in Klammern: nach Mglichkeit keine direkten Zugriffe
; OS: vom ES4.0 in einigen Routinen genutzt
; OSD: vom ES4.0 stndig genutzt
; %0000-07FF interner ROM des U883
; %0800-1FFF Betriebssystem ES4.0 auf EPROM 2764
; %2000-3FFF frei
; %4000-5FFF Video-RAM-bnke
; %6000-63FF Steuerregister fr Videorambankauswahl
; %6400-7BFF frei (fr IO-Gerte, Ramdisk, ...)
; %7000-7FFF Tastaturabfrage
; %8000- RAM-Beginn bei 32 KByte Vollausbau
; %C000- RAM-Beginn bei 16 KByte
; %E000- RAM-Beginn bei 8 KByte
; bis F4FF RAM, frei zur Nutzung
; %F500-F511 IRQ-Sprnge
; %F512-F579 RAM fr Druckroutine, siehe 8.
; (%F57B-F5FF Kassettenpuffer)
; %F600-F6FF Stack
; %F700-F74F Zeilenspeicher bei Rckkehr aus Routine CHARIN
; (%F750-F76F RAM fr Gleitkommaroutinen)
; %F770-F77F noch frei
; (%F780-F78F Registerspeicher des MON)
; %F790-F79F noch frei
; %F7A0 Bitmaske fr Textzeichen, siehe 11.
; %F7A1 Bitmaske fr Kursor, siehe 11.
; (%F7A2-F7AB OSD)
; (%F7AC-F7DF evtl. spter OSD)
; %F7E0-F7FF Funktionstastenprogramme
; (%F800-FBFF zweiter ASCII-Textspeicher)
; (%FC00-FFBF ASCII-Textspeicher)
; %FFC0-FFFF IO-Vektortabelle, siehe 16
; 
; %F512 Drucker-Routine
; 
; Aufzeichnungsverfahren
; Fileaufbau: 5s Vorton, Blcke mit Pause von 40 ms
; Blocksendung: Vorimpuls (eine Phase mit 0.5 ms), %85 Bytes
; Blockaufbau: (Adresse im Kassettenpuffer als erste Zahl)
; %F57B Blocknummer %00 bis %FF, evt. wieder von vorn
; %F57C Kopie von %F57B
; %F57D Prfsumme ber Formatbyte und Datenbytes (Byte-Summe mit dazuaddierten Carrys)
; %F57E Kopie von %F57D
; %F57F Formatbyte: %FA: weniger als 128 Datenbyte. Anzahl der gltigen Datenbyte auf %F5FF
; 	%FC: 128 Datenbyte
; 	%FE: End of File Block, ohne sinnvolle Datenbytes
; %F580-F5FF Datenbytes

;------------------------------------------------------------------------------
; System-Register
;------------------------------------------------------------------------------

p0 equ	0				; Port 0
p1 equ	1				; Port 1
p2 equ	2				; Port 2
p3 equ	3				; Port 3

sio equ 0F0h				; Serial I/O
tmr equ 0F1h				; Timer	mode
t1 equ	0F2h				; Timer/counter	1
pre1 equ 0F3h				; T1 prescaler
t0 equ	0F4h				; Timer/counter	0
pre0 equ 0F5h				; T0 prescaler
p2m equ 0F6h				; Port 2 mode
p3m equ 0F7h				; Port 3 mode
p01m equ 0F8h				; Ports	0-1 mode
ipr equ 0F9h				; Interrupt priority register
irq equ 0FAh				; Interrupt request register
imr equ 0FBh				; Interrupt mask register
flags equ 0FCh				; Program control flags
rp equ	0FDh				; Register pointer
gpr equ 0FEh				; General purpose register
spl equ 0FFh				; Stack	pointer

;------------------------------------------------------------------------------
; 
;------------------------------------------------------------------------------

		org 800h
		
		jp	0F500h		; IRQ 0
		jp	0F503h		; IRQ 1
		jp	0F506h		; IRQ 2
		jp	0F509h		; IRQ 3
		jp	@word_56	; IRQ 4
		nop
		jp	0F50Fh		; IRQ 5
; Sprungtabelle
KOMMAND:	jp	begin		; Eintritt aus Bootstrap Module
CHARIN:		jp	0FFE6h		; k2_get -> lCHARIN -> (CHARIN0 bzw. WKEY)
CHAROUT:	jp	0FFE9h		; k2_put -> lCHAROUT -> (co)

KEY:		jp	key0
WKEY:		jp	wkey0
SAVE:		jp	save0
LOAD:		jp	load0
SCRFUN:		jp	scrfun0
MONITOR:	jp	monitor0
PRISTRI:	jp	pristri0	; Textausgabe
SHOWPLAYER:	jp	showplayer0
HIDEPLAYER:	jp	hideplayer0
RND:		jp	rnd0

		nop

;------------------------------------------------------------------------------
; setzt den Punkt (X,Y) mit der Farbe aus Z
;------------------------------------------------------------------------------

g_plot:		push	rp		; Register pointer
		srp	#60h
		call	sub_FBB
		ld	R10, var_Z_lo
		ld	R6, #0FFh
		xor	R6, R3
		ld	R2, #0
		ld	R4, #60h
		ld	R5, #0EFh
		ld	p01m, #0B2h	; Ports	0-1 mode
loc_850:	tm	p3, #4		; Port 3
		jr	NZ, loc_850
loc_855:	lde	@RR4, R5
		ld	R8, #31h
		rrc	R10
		rlc	R8
		lde	R9, @RR0
		and	R9, R6
		or	R9, @R8
		lde	@RR0, R9
		rl	R5
		jr	C, loc_855
		ld	p01m, #92h	; Ports	0-1 mode
		pop	rp		; Register pointer
		ret
		
		db 0FFh

;------------------------------------------------------------------------------
; Einsprung nach Reset
;------------------------------------------------------------------------------

begin:		srp	#0F0h
		ld	R14, #0F7h
		ld	R15, #0
		ld	R8, #92h
		srp	#50h
		ld	R4, #0
		ld	R5, #57h
		ld	R6, #1Ch	; p_irq1
		ld	R7, #0Eh
		call	sub_910
		srp	#60h
		ld	R7, #10h
		ld	R12, #68h
		ld	R0, #0F7h
		ld	R1, #0A0h
		ld	R2, #2Dh
		ld	R3, #0C3h
		lde	@RR0, R2
		inc	R1
		lde	@RR0, R3
		; Sprungtabelle ans RAM-Ende kopieren
		ld	R0, #hi(kanal_sv)
		ld	R1, #lo(kanal_sv)
		ld	R2, #0FFh
		ld	R3, #0E0h
loc_8A0:	lde	R4, @RR0
		lde	@RR2, R4
		inc	R1
		inc	R3
		jr	NZ, loc_8A0
		;
		clr	word_8
		clr	byte_7
		ld	byte_6,	#0E0h
		call	PRISTRI		; Textausgabe
		db 0Ch,"Willkommen am JTC : ES4.0 by SWB 1990",0Dh,0
		nop
		nop
		nop
		jp	EDI

;------------------------------------------------------------------------------
; SCRFUN sind drei Spezialfunktionen fr den Textbildschirm, die
; zur Datenbergabe die Basicvariable X, Y, Z nutzen. Es wird aber nur
; das Lowbyte beachtet und gesetzt, also Variable X entspricht
; Register %4F. Y:%51, Z:%53. Die Funktionen werden anhand des in %53
; zu bergebenen Wertes (der bei Rckkehr auf 00 steht wenn nicht
; anders vermerkt) unterschieden. Der Textbildschirm hat ein
; Koordinatensystem, dessen Ursprung (0,0) in der linken oberen Ecke
; liegt. X luft nach rechts bis 39 und Y nach unten bis 23.
; %00 liefert in %4F (Spalte) und in %51 (Zeile) die Position des
; Textkursors.
; %01 setzt den Textkursor auf die in %4F (Spalte) und %51 (Zeile)
; bergebene Position, falls diese im Bereich des Machbaren liegt.
; %02 liefert in %53 den ASCII-Kode des Zeichens, das sich auf der in
; %4F und %51 bergebenen Position befindet. Wer hier unsinnige
; Positionen angibt, bekommt auch Mll zurck.
;------------------------------------------------------------------------------

scrfun0:	push	rp		; Register pointer
		srp	#50h
		or	R3, R3
		jr	NZ, loc_8EF
; fkt 0, get cupos
		ld	var_Y_lo, R12	
		ld	var_X_lo, R11
loc_8EC:	pop	rp		; Register pointer
		ret
loc_8EF:	djnz	R3, loc_901
; fkt 1, set cupos
		cp	var_Y_lo, #18h	
		jr	NC, loc_8EC
		cp	var_X_lo, #28h
		jr	NC, loc_8EC
		ld	R12, var_Y_lo
		ld	R11, var_X_lo
		jr	loc_8EC
loc_901:	djnz	R3, loc_8EC
; fkt 2, get char
		push	cupos		
		push	byte_5C
		ld	R12, var_Y_lo
		ld	R11, var_X_lo
		srp	#60h
		jr	loc_922
		
		db 0FFh

;------------------------------------------------------------------------------
; 
;------------------------------------------------------------------------------

sub_910:	srp	#0F0h
		ld	R11, #10h
		ld	R9, #8
		ld	R8, #92h
		ld	R7, #1
		ld	R5, #41h
		ld	R4, #0FAh
		ld	R1, #3
		ei
		ret

;------------------------------------------------------------------------------
; 
;------------------------------------------------------------------------------

loc_922:	call	sub_1856
		lde	R2, @RR0
		ld	var_Z_lo, R2
		pop	byte_5C
		pop	cupos
		jr	loc_8EC

		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh

;------------------------------------------------------------------------------
; 
;------------------------------------------------------------------------------

EDI:		call	PRISTRI		; Textausgabe
		db "Edi",0Dh,0

kdoloop:	or	byte_55, #40h
		ld	byte_58, #0FFh
		srp	#10h
		call	CHARIN
		cp	byte_13, #0Dh
		jr	Z, EDI
		ld	R6, byte_13
;?X	druckt Wert der	Variable X aus
		cp	R6, #3Fh
		jr	NZ, kdo2
		call	CHARIN
		ld	R5, byte_13
		ld	R6, byte_13
		rl	byte_16
		ld	R2, 9Eh(R6)
		ld	R3, 9Fh(R6)
		call	CHAROUT
		ld	R5, #3Dh
		call	CHAROUT
		call	PrDez
		ld	R5, #0Dh
		call	CHAROUT
		jr	kdoloop
kdo2:		cp	R6, #43h
		jr	NZ, kdo3
		tm	byte_F,	#8
		jr	Z, kdo7
		and	byte_F,	#0F7h
		jr	kdo6
kdo3:		ld	byte_F,	#4
		clr	byte_E
		ld	R0, byte_6
		ld	R1, byte_7
		cp	R6, #4Eh
		jr	NZ, kdo5
		ld	R4, #0
		lde	@RR0, R4
		jr	EDI
kdo5:		cp	R6, #52h
		jr	NZ, kdo9
		call	6C9h
kdo6:		and	byte_55, #0BFh
		ld	byte_58, #0FFh
		call	738h
		or	byte_55, #40h
		ld	R5, #4Fh
		tm	byte_F,	#2
		jr	NZ, kdo8
		ld	R5, #53h
		tm	byte_F,	#8
		jr	NZ, kdo8
kdo7:		ld	R5, #45h
kdo8:		call	CHAROUT
		ld	R3, #0F0h
		and	byte_13, byte_E
		swap	byte_13
loc_9DA:	clr	byte_12
		call	PrDez
		jp	EDI
;
kdo9:		cp	R6, #'E'
		jp	Z, list
		cp	R6, #'S'
		jr	NZ, loc_A0C
		call	sub_EC5
		ld	byte_20, R0
		ld	byte_21, R1
		sub	R3, R1
		sbc	R2, R0
		incw	RR2
		ld	unk_22,	R2
		ld	unk_23,	R3
		ld	byte_60, #0F7h
		ld	unk_61,	#1
		call	SAVE
loc_A07:	ld	R3, byte_24
		jp	loc_9DA
loc_A0C:	cp	R6, #'L'
		jr	NZ, loc_A20
		ld	byte_20, R0
		ld	byte_21, R1
		ld	byte_60, #0F7h
		ld	unk_61,	#1
		call	LOAD
		jr	loc_A07
loc_A20:	cp	R6, #'M'
		jp	NZ, kdo15
		call	MONITOR
		or	byte_55, #40h
		jp	EDI
		nop

;------------------------------------------------------------------------------
; 
;------------------------------------------------------------------------------

lCHAROUT:	call	co
		rcf
		ret

;------------------------------------------------------------------------------
; 
;------------------------------------------------------------------------------

lCHARIN:	tm	byte_55, #4
		jr	Z, loc_A3F
		call	CHARIN0
		rcf
		ret
loc_A3F:	call	WKEY
		ld	unk_5A,	byte_13
		call	co2
		rcf
		ret
		
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh

;------------------------------------------------------------------------------
; wandelt eine 1 bis 5 Ziffern lange Dezimalzahl (0 bis
; 65535) ab (01Eh/1F) nach 014h/15, 012h/13 und 01Ch/1D zerstrt, 01Fh um
; Anzahl der gewandelten Ziffern erhht
;------------------------------------------------------------------------------

DAXTH16:	ld	R12, #5
		ld	R4, #0
		ld	R5, #0
loc_A58:	lde	R13, @RR14
		sub	R13, #30h
		jr	C, loc_A77
		cp	R13, #0Ah
		jr	NC, loc_A77
		ld	R10, #9
		ld	R2, R4
		ld	R3, R5
loc_A6A:	add	R5, R3
		adc	R4, R2
		djnz	R10, loc_A6A
		add	R5, R13
		adc	R4, R10
		inc	R15
		djnz	R12, loc_A58
loc_A77:	rcf
		ret

;------------------------------------------------------------------------------
; 
;------------------------------------------------------------------------------

list:		ld	R15, #1
		ld	R14, #0F7h
		call	DAXTH16
		or	R4, #80h
list1:		lde	R10, @RR0
		incw	RR0
		lde	R11, @RR0
		decw	RR0
		or	R10, R10
		jr	Z, list4
		cp	R10, R4
		jr	C, list2
		jr	UGT, list5
		cp	R11, R5
		jr	Z, list5
		jr	UGT, list5
list2:		incw	RR0
list3:		incw	RR0
		lde	R10, @RR0
		cp	R10, #0Dh
		jr	NZ, list3
		incw	RR0
		jr	list1
list4:		jp	kdoloop
list5:		lde	R2, @RR0
		tm	R2, #80h
		jr	Z, list4
		incw	RR0
		lde	R3, @RR0
		and	R2, #7Fh
		call	PrDez
list6:		incw	RR0
		lde	R5, @RR0
		cp	R5, #0Dh
		jr	Z, list7
		or	byte_55, #80h
list7:		call	CHAROUT
		cp	R5, #0Dh
		jr	NZ, list6
		call	WKEY
		cp	R3, #20h
		jr	NZ, list4
		incw	RR0
		jr	list5
		
		nop
		nop

;------------------------------------------------------------------------------
; Faaaa aaaa bc fllt Speicher ab Adresse aaaa mit aaaa Bytes
; bc, mon
;------------------------------------------------------------------------------

mon_fill:	call	ADRE		; Monitor Fill
		ld	R8, R10
		ld	R9, R11
		call	ADRE		; Wandlung Adresse
loc_AEA:	call	ATH8		; Ascii	To Hex 2 Ziffern
		jr	C, locret_AFF
fill1:		lde	@RR8, R13
		incw	RR8
		decw	RR10
		jr	NZ, fill1

;------------------------------------------------------------------------------
; druckt "Mon" und einen Returncode aus. Wenn ein Befehl
; nach erfolgreicher Ausfhrung "Mon" melden soll, dann ist er statt
; mit RET mit JP 00AF7h zu beenden.
;------------------------------------------------------------------------------

PMON:		call	PRISTRI		; Textausgabe
		db "Mon",0Dh,0
locret_AFF:	ret

;------------------------------------------------------------------------------
; 
;------------------------------------------------------------------------------
; muss xx00-Adr. sein
		align 100h

pixtab:		db	 0
		db  20h
		db  40h ; @
		db  60h ; `
		db  80h ; 
		db 0A0h ; 
		db	 1
		db  21h ; !
		db  41h ; A
		db  61h ; a
		db  81h ; 
		db 0A1h ; 
		db	 2
		db  22h ; "
		db  42h ; B
		db  62h ; b
		db  82h ; 
		db 0A2h ; 
		db	 3
		db  23h ; #
		db  43h ; C
		db  63h ; c
		db  83h ; 
		db 0A3h ; 
		db	 4
		db  24h ; $
		db  44h ; D
		db  64h ; d
		db  84h ; 
		db 0A4h ; 
		db	 5
		db  25h ; %
		db  45h ; E
		db  65h ; e
		db  85h ; 
		db 0A5h ; 
		db	 6
		db  26h ; &
		db  46h ; F
		db  66h ; f
		db  86h ; 
		db 0A6h ; 
		db	 7
		db  27h ; '
		db  47h ; G
		db  67h ; g
		db  87h ; 
		db 0A7h ; 
		db	 8
		db  28h ; (
		db  48h ; H
		db  68h ; h
		db  88h ; 
		db 0A8h ; 
		db	 9
		db  29h ; )
		db  49h ; I
		db  69h ; i
		db  89h ; 
		db 0A9h ; 
		db  0Ah
		db  2Ah ; *
		db  4Ah ; J
		db  6Ah ; j
		db  8Ah ; 
		db 0AAh ; 
		db  0Bh
		db  2Bh ; +
		db  4Bh ; K
		db  6Bh ; k
		db  8Bh ; 
		db 0ABh ; 
		db  0Ch
		db  2Ch ; ,
		db  4Ch ; L
		db  6Ch ; l
		db  8Ch ; 
		db 0ACh ; 
		db  0Dh
		db  2Dh ; -
		db  4Dh ; M
		db  6Dh ; m
		db  8Dh ; 
		db 0ADh ; 
		db  0Eh
		db  2Eh ; .
		db  4Eh ; N
		db  6Eh ; n
		db  8Eh ; 
		db 0AEh ; 
		db  0Fh
		db  2Fh ; /
		db  4Fh ; O
		db  6Fh ; o
		db  8Fh ; 
		db 0AFh ; 
		db  10h
		db  30h ; 0
		db  50h ; P
		db  70h ; p
		db  90h ; 
		db 0B0h ; 
		db  11h
		db  31h ; 1
		db  51h ; Q
		db  71h ; q
		db  91h ; 
		db 0B1h ; 
		db  12h
		db  32h ; 2
		db  52h ; R
		db  72h ; r
		db  92h ; 
		db 0B2h ; 
		db  13h
		db  33h ; 3
		db  53h ; S
		db  73h ; s
		db  93h ; 
		db 0B3h ; 
		db  14h
		db  34h ; 4
		db  54h ; T
		db  74h ; t
		db  94h ; 
		db 0B4h ; 
		db  15h
		db  35h ; 5
		db  55h ; U
		db  75h ; u
		db  95h ; 
		db 0B5h ; 
		db  16h
		db  36h ; 6
		db  56h ; V
		db  76h ; v
		db  96h ; 
		db 0B6h ; 
		db  17h
		db  37h ; 7
		db  57h ; W
		db  77h ; w
		db  97h ; 
		db 0B7h ; 
		db  18h
		db  38h ; 8
		db  58h ; X
		db  78h ; x
		db  98h ; 
		db 0B8h ; 
		db  19h
		db  39h ; 9
		db  59h ; Y
		db  79h ; y
		db  99h ; 
		db 0B9h ; 
		db  1Ah
		db  3Ah ; :
		db  5Ah ; Z
		db  7Ah ; z
		db  9Ah ; 
		db 0BAh ; 
		db  1Bh
		db  3Bh ; ;
		db  5Bh ; [
		db  7Bh ; {
		db  9Bh ; 
		db 0BBh ; +
		db  1Ch
		db  3Ch ; <
		db  5Ch ; 
		db  7Ch ; |
		db  9Ch ; 
		db 0BCh ; +
		db  1Dh
		db  3Dh ; =
		db  5Dh ; ]
		db  7Dh ; }
		db  9Dh ; 
		db 0BDh ; 
		db  1Eh
		db  3Eh ; >
		db  5Eh ; ^
		db  7Eh ; ~
		db  9Eh ; 
		db 0BEh ; 
		db  1Fh
		db  3Fh ; ?
		db  5Fh ; _
		db  7Fh ; 
		db  9Fh ; 
		db 0BFh ; +
;
		db	 0
		db  28h ; (
		db  50h ; P
		db  80h ; 
		db 0A8h ; 
		db 0D0h ; 
		db 0FFh
		db 0FFh
		db  80h ; 
		db	 8
		db  20h
		db	 2
		db  40h ; @
		db	 4
		db  10h
		db	 1

;------------------------------------------------------------------------------
; Monitor Registeranzeige
; Rbc zeigt das Register bc an, spc
;------------------------------------------------------------------------------

mon_regs:	call	ATH8		; Ascii	To Hex 2 Ziffern
		jr	C, locret_BE4
		ld	R10, R13
loc_BD7:	ld	R11, @R10
		ld	R5, #'!'
		call	PCAS		; print	char 015h, adr 018h/019h + space
		inc	R10
		call	RWCONT		; Enter+Warten
		jr	Z, loc_BD7
locret_BE4:	ret

;------------------------------------------------------------------------------
; !bcbc Register bc wird mit Byte bc beschrieben, wenn
; Byte bc korrekt war (also eine Hexzahl), dann wird nchste
; Registeradresse ausgegeben. (zwischen den beiden bc steht wirklich
; kein Leerzeichen)
;------------------------------------------------------------------------------

mon_regwr:	call	ADRE		; Monitor Register Schreiben
		ld	@R10, R11
		inc	R10
		ld	R5, #'!'
		call	CHAROUT
		ld	R9, R10
		jp	HTA8		; druckt 019h als 2-Ziffer-Hexzahl aus
loc_BF5:	jp	loc_F37

;------------------------------------------------------------------------------
; 
;------------------------------------------------------------------------------

monitor0:	call	PMON
		push	rp		; Register pointer
		srp	#60h
		ld	R0, #0F7h
		ld	R1, #80h
		ld	R2, #10h
		ld	R3, #10h
mon1:		ldei	@RR0, @R2
		djnz	R3, mon1
		srp	#10h
mon2:		ld	byte_58, #0FFh
		and	byte_55, #0BFh
		call	CHARIN
		ld	R15, #0
		ld	R14, #0F7h
		lde	R0, @RR14
		inc	R15
		ld	R12, #0Ch	; kdotab
		ld	R13, #39h
		ld	R1, #10h
mon3:		lde	R2, @RR12
		inc	R13
		cp	R2, R0
		jr	NZ, mon4
		lde	R0, @RR12
		inc	R13
		lde	R1, @RR12
		call	@unk_10
		jr	mon2
mon4:		inc	R13
		inc	R13
		djnz	R1, mon3
		jr	loc_BF5

;------------------------------------------------------------------------------
; Monitorkommandos
;------------------------------------------------------------------------------

kdotab:		db  'F'
		dw mon_fill		; Monitor Fill
		db  'R'
		dw mon_regs
		db  'H'
		dw mon_hexanz	; Monitor Hexanzeige
		db  ','
		dw mon_hexwrite	; Monitor Speicher beschreiben
		db  'J'
		dw mon_jump		; Monitor Jump
		db  'A'
		dw mon_ascii		; Monitor ASCII-Anzeige
		db  ';'
		dw mon_asciiwrite	; Monitor ASCII	Schreiben
		db  'S'
		dw mon_save		; Monitor Speichern
		db  'L'
		dw mon_load		; Monitor Laden
		db  'M'
		dw mon_move		; Monitor Move
		db  '?'
		dw mon_arith		; Monitor Arithmetik
		db  'Q'
		dw mon_quit		; Monitor Rckkehr
		db  '!'
		dw mon_regwr		; Monitor Register Schreiben
		db  '#'
		dw mon_dezhex		; Monitor Dez>Hex
		db  '%'
		dw mon_hexdez		; Monitor Hex>Dez
		db 0FFh
		db 0FFh
		db 0FFh

;------------------------------------------------------------------------------
; druckt 018h/019h als 4-Ziffer-Hexzahl aus
;------------------------------------------------------------------------------

HTA16:		push	R9
		ld	byte_19, R8
		call	HTA8		; druckt 019h als 2-Ziffer-Hexzahl aus
		pop	R9

;------------------------------------------------------------------------------
; druckt 019h als 2-Ziffer-Hexzahl aus
;------------------------------------------------------------------------------

HTA8:		push	R9
		swap	R9
		call	HTA4		; druckt die niederen 4	Bit aus	019h als	Hexziffer aus
		pop	R9

;------------------------------------------------------------------------------
; druckt die niederen 4	Bit aus	019h als Hexziffer aus
;------------------------------------------------------------------------------

HTA4:		ld	R5, #0Fh
		and	R5, R9
		add	R5, #30h
		cp	R5, #3Ah
		jr	C, hta4_1
		add	R5, #7
hta4_1:		jp	CHAROUT

;------------------------------------------------------------------------------
; druckt einen Return-Kode aus (00Dh)
;------------------------------------------------------------------------------

PRRET:		ld	R5, #0Dh
		jr	hta4_1

;------------------------------------------------------------------------------
; druckt einen Return-Kode aus und wartet dann auf
; Tastendruck, wenn die Leertaste gedrckt wurde ist das Z-Flag
; gesetzt
; Enter+Warten
;------------------------------------------------------------------------------

RWCONT:		call	PRRET		; druckt einen Return-Kode aus (00Dh)
		call	WKEY
		jp	loc_DFA

		db 0FFh

;------------------------------------------------------------------------------
; druckt das Zeichen aus 015h aus, dann die 16 Bit aus
; 01Ah/1B (018h/19 zerstrt) als Hexzahl und noch ein Leerzeichen
; print char 015h, adr 018h/019h + space
;------------------------------------------------------------------------------

PCAS:		call	CHAROUT
		ld	byte_18, R10
		ld	byte_19, R11
		call	HTA16		; druckt 018h/019h als 4-Ziffer-Hexzahl aus
		ld	R5, #20h
		jr	hta4_1

;------------------------------------------------------------------------------
; wandelt die 4 Ziffern ab Adresse (01Eh/1F) nach 01Ch/1D und
; %lA/1B, bei Wandlungsfehler wird POP POP RET ausgefhrt, sonst
; Rckkehr mit um fnf erhhtem 01Fh
; Wandlung Adresse
;------------------------------------------------------------------------------

ADRE:		call	ATH16		; Ascii	To Hex 4 Ziffern
		jr	NC, adre1
		pop	R0
		pop	R0
adre1:		ld	R10, R12
		ld	R11, R13
		inc	R15
		ret

;------------------------------------------------------------------------------
; wandelt die Ziffer von (01Eh/1F) in niedere 4 Bit von 01Dh,
; 01Fh um eins erhht
; Ziffer 01Eh/01Fh nach Hex 01Dh (low nibble)
;------------------------------------------------------------------------------

ATH4:		lde	R13, @RR14
		inc	R15
		cp	R13, #30h
		jr	C, ath4_2
		sub	R13, #30h
		cp	R13, #0Ah
		jr	C, ath4_1
		cp	R13, #11h
		jr	C, ath4_2
		sub	R13, #7
		cp	R13, #10h
ath4_1:		ccf
ath4_2:		ret

;------------------------------------------------------------------------------
; wandelt 4 Ziffern ab (01Eh/1F) nach 01Ch/1D, 012h zerstrt,
; 01Fh um vier erhht
; Ascii To Hex 4 Ziffern
;------------------------------------------------------------------------------

ATH16:		call	ATH8		; Ascii	To Hex 2 Ziffern
		jr	C, ath8_1
		ld	word_1C, R13

;------------------------------------------------------------------------------
; wandelt 2 Ziffern ab (01Eh/1F) nach 01Dh, 012h zerstrt, 01Fh
; um zwei erhht
; Ascii To Hex 2 Ziffern
;------------------------------------------------------------------------------

ATH8:		call	ATH4		; Ziffer 01Eh/01Fh nach Hex 01Dh (low nibble)
		jr	C, ath8_1
		ld	R2, R13
		swap	R2
		call	ATH4		; Ziffer 01Eh/01Fh nach Hex 01Dh (low nibble)
		or	R13, R2
ath8_1:		ret

;------------------------------------------------------------------------------
; Rckkehr zum aufrufenden Programm (meist EDI)
;------------------------------------------------------------------------------

mon_quit:	pop	R0		; Monitor Rckkehr
		pop	R0
		jp	quit1

;------------------------------------------------------------------------------
; Hexanzeige 8 aufeinanderfolgender
; Speicherstellen, spc
;------------------------------------------------------------------------------

mon_hexanz:	call	ADRE		; Monitor Hexanzeige
hex1:		ld	R5, #','
		call	PCAS		; print	char 015h, adr 018h/019h +	space
		ld	R0, #8
hex2:		lde	R9, @RR10
		incw	RR10
		call	HTA8		; druckt 019h als 2-Ziffer-Hexzahl aus
		ld	R5, #' '
		call	CHAROUT
		djnz	R0, hex2
		call	RWCONT		; Enter+Warten
		jr	Z, hex1
hex4:		ret

;------------------------------------------------------------------------------
; ,aaaa bc bc bc bc bc bc bc bc Hexschreiben des Speichers, auch bei
; weniger als 8 Byte bc werden Daten eingeschrieben, aber keine neue
; Adresse ausgegeben
;------------------------------------------------------------------------------

mon_hexwrite:	call	ADRE		; Monitor Speicher beschreiben
		ld	R0, #8
hexw1:		call	ATH8		; Ascii	To Hex 2 Ziffern
		jr	C, hex4
		inc	R15
		lde	@RR10, R13
		incw	RR10
		djnz	R0, hexw1
		ld	R5, #','
hexw3:		jp	PCAS		; print	char 015h, adr 018h/019h +	space

;------------------------------------------------------------------------------
; ASCII-Anzeige von 16 aufeinanderfolgenden
; Speicherstellen, spc
;------------------------------------------------------------------------------

mon_ascii:	call	ADRE		; Monitor ASCII-Anzeige
ascii1:		ld	R5, #';'
		call	PCAS		; print	char 015h, adr 018h/019h +	space
		ld	R0, #10h
ascii2:		lde	R5, @RR10
		or	byte_55, #80h
		incw	RR10
		call	CHAROUT
		djnz	R0, ascii2
		call	RWCONT		; Enter+Warten
		jr	Z, ascii1
		ret

;------------------------------------------------------------------------------
; Jaaaa startet Maschinenprogramm ab Adresse aaaa mit
; CALL, das Programm kann mit RET zum MON zurckkehren, das whrend
; des Programmlaufes verwendete Nutzer-Flagregister ist sonst in
; Register 016h gespeichert und der Nutzer-Registerpointer in 017h, mon
;------------------------------------------------------------------------------

mon_jump:	call	ADRE		; Monitor Jump
		ld	flags, R6	; Program control flags
		ld	rp, byte_17	; Register pointer
		call	@unk_1A
		ld	byte_17, rp	; Register pointer
		srp	#10h
		ld	R6, flags	; Program control flags
		jp	PMON

;------------------------------------------------------------------------------
; Saaaa aaaa speichert ab Adresse aaaa auf Kassette aaaa
; Bytes, gibt danach 00 als in Ordnung-Meldung aus, mon
; Im Unterschied zum 4K-Betriebssystem ist als zweite Zahl nicht die
; Endadresse, sondern die Anzahl der abzuspeichernden Bytes anzugeben.
;------------------------------------------------------------------------------

mon_save:	call	ADRE		; Monitor Speichern
		ld	byte_20, R10
		ld	byte_21, R11
		call	ADRE		; Wandlung Adresse
		ld	unk_22,	R10
		ld	unk_23,	R11
		ld	byte_60, R14
		ld	unk_61,	R15
		call	SAVE
		jr	loc_DAD

		db 0FFh
		db 0FFh

;------------------------------------------------------------------------------
; ;aaaa dddddddddddddddd ASCII-Schreiben, es werden die Zeichen
; bis einschlielich eines RET-Zeichens, aber maximal 16, in den
; Speicher geschrieben. Wenn 16 Zeichen geschrieben wurden und auch
; das 16. kein RET-Zeichen war, dann wird die nchste Adresse
; automatisch ausgegeben.
;------------------------------------------------------------------------------

mon_asciiwrite:	call	ADRE		; Monitor ASCII	Schreiben
		ld	R0, #10h
asciiw1:	lde	R2, @RR14
		lde	@RR10, R2
		cp	R2, #0Dh
		jr	Z, locret_D86
		inc	R15
		incw	RR10
		djnz	R0, asciiw1
		ld	R5, #';'
		jr	hexw3
locret_D86:	ret
		
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh

;------------------------------------------------------------------------------
; Laaaa ldt von Kassette in Speicher ab Adresse aaaa,
; druckt dann aaaa bc aus, aaaa ist geladene Byteanzahl und bc
; Fehlermeldung (00: fehlerfrei), mon
;------------------------------------------------------------------------------

mon_load:	call	ADRE		; Monitor Laden
		ld	byte_20, R10
		ld	byte_21, R11
		ld	byte_60, R14
		ld	unk_61,	R15
		call	LOAD
		ld	R8, unk_22
		ld	R9, unk_23
		call	HTA16		; druckt 018h/019h als 4-Ziffer-Hexzahl aus
		ld	R5, #' '
		call	CHAROUT
loc_DAD:	ld	R9, byte_24
		call	HTA8		; druckt 019h als 2-Ziffer-Hexzahl aus
mload1:		jp	PMON
quit1:		srp	#60h
		ld	R0, #0F7h
		ld	R1, #80h
		ld	R2, #10h
		ld	R3, #10h
quit2:		ldei	@R2, @RR0
		djnz	R3, quit2
		pop	rp		; Register pointer
		ret

;------------------------------------------------------------------------------
; ?aaaa aaaa gibt aaaa plus aaaa.und aaaa minus aaaa aus
;------------------------------------------------------------------------------

mon_arith:	call	ADRE		; Monitor Arithmetik
		call	ATH16		; Ascii	To Hex 4 Ziffern
		jr	C, arith1
		ld	R8, R10
		ld	R9, R11
		add	R9, R13
		adc	R8, R12
		call	HTA16		; druckt 018h/019h als 4-Ziffer-Hexzahl aus
		ld	R5, #' '
		sub	R11, R13
		sbc	R10, R12
		call	PCAS		; print	char 015h, adr 018h/019h +	space
		jr	mload1
arith1:		ret

;------------------------------------------------------------------------------
; #eeeee gibt die vorzeichenlose Dezimalzahl eeeee (O bis
; 65635) umgewandelt in hex aus
;------------------------------------------------------------------------------

mon_dezhex:	call	DAXTH16		; Monitor Dez>Hex
		ld	R10, byte_14
		ld	R11, byte_15
		ld	R5, #25h
		call	PCAS		; print	char 015h, adr 018h/019h +	space
		jp	PRRET		; druckt einen Return-Kode aus (00Dh)

		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh

;------------------------------------------------------------------------------
; 
;------------------------------------------------------------------------------

loc_DFA:	cp	byte_13, #20h
		jr	Z, locret_E06
		ld	unk_5A,	R3
		call	co2
		or	R3, R3
locret_E06:	ret

		db 0FFh
		db 0FFh

;------------------------------------------------------------------------------
; Zeilennummer,	FSE
;------------------------------------------------------------------------------

kdo15:		cp	R6, #3Ah	; '9'+1
kdo16:		jp	NC, kdoloop
		cp	R6, #30h	; '0'
		ccf
		jr	NC, kdo16
		ld	R15, #0
		ld	R14, #0F7h
		call	DAXTH16
		ld	byte_60, R15
		ld	unk_61,	R15
		com	unk_61
		add	unk_61,	#5
		nop
		nop
		nop
		or	R4, #80h
		call	sub_EC5
;
inp1:		lde	R10, @RR0
		incw	RR0
		lde	R11, @RR0
		decw	RR0
		or	R10, R10
		jr	Z, inp9
		cp	R10, R4
		jr	C, inp2
		jr	UGT, inp9
		cp	R11, R5
		jr	Z, inp4
		jr	UGT, inp9
inp2:		incw	RR0
inp3:		incw	RR0
		lde	R10, @RR0
		cp	R10, #0Dh
		jr	NZ, inp3
		incw	RR0
		jr	inp1
inp4:		ld	R6, R0
		ld	R7, R1
		incw	RR6
inp5:		incw	RR6
		lde	R10, @RR6
		cp	R10, #0Dh
		jr	NZ, inp5
		incw	RR6
		ld	R8, R0
		ld	R9, R1
inp6:		lde	R10, @RR6
		lde	@RR8, R10
		cp	R6, R2
		jr	NZ, inp7
		cp	R7, R3
		jr	Z, inp8
inp7:		incw	RR6
		incw	RR8
		jr	inp6
inp8:		ld	R2, R8
		ld	R3, R9
inp9:		dec	R15
		ld	R14, byte_58
		cp	R14, R15
		jp	LE, kdoloop
		ld	R7, unk_61
		add	R7, R14
		ld	R6, #0
		add	R7, R3
		adc	R6, R2
inp10:		lde	R10, @RR2
		lde	@RR6, R10
		cp	R0, R2
		jr	NZ, inp11
		cp	R1, R3
		jr	Z, inp12
inp11:		decw	RR2
		decw	RR6
		jr	inp10
inp12:		lde	@RR0, R4
		incw	RR0
		lde	@RR0, R5
		incw	RR0
		ld	R2, #0F7h
		ld	R3, byte_60
		sub	R14, R15
inp13:		lde	R10, @RR2
		lde	@RR0, R10
		incw	RR0
		inc	R3
		dec	byte_1E
		jr	NZ, inp13
		ld	R10, #0Dh
		lde	@RR0, R10
		jp	kdoloop

;------------------------------------------------------------------------------
; 
;------------------------------------------------------------------------------

sub_EC5:	ld	R2, R0
		ld	R3, R1
loc_EC9:	lde	R10, @RR2
		tm	R10, #80h
		jr	Z, locret_EDF
		incw	RR2
loc_ED2:	incw	RR2
		lde	R10, @RR2
		cp	R10, #0Dh
		jr	NZ, loc_ED2
		incw	RR2
		jr	loc_EC9
locret_EDF:	ret

;------------------------------------------------------------------------------
; 
;------------------------------------------------------------------------------

PrDez:		call	182h
		cp	R4, #20h
		jr	Z, prDez1
		push	byte_15
		ld	R5, R4
		call	CHAROUT
		pop	byte_15
prDez1:		ld	R11, #15h
loc_EF3:		cp	@unk_1B, #30h
		jr	NZ, prDez2
		inc	R11
		cp	R11, #19h
		jr	C, loc_EF3
prDez2:		ld	R5, @R11
		call	CHAROUT
		inc	R11
		cp	R11, #1Ah
		jr	C, prDez2
		ret

;------------------------------------------------------------------------------
; liefert die Farbe des Punktes (X,Y) in Z
;------------------------------------------------------------------------------

g_ptest:	push	rp		; Register pointer
		srp	#60h
		call	sub_FBB
		clr	var_Z_lo
		ld	R4, #60h
		ld	R5, #0EFh
		ld	p01m, #0B2h	; Ports	0-1 mode
loc_F1A:	tm	p3, #4		; Port 3
		jr	NZ, loc_F1A
loc_F1F:	lde	@RR4, R5
		lde	R9, @RR0
		rcf
		tm	R9, R3
		jr	Z, loc_F29
		scf
loc_F29:	rrc	var_Z_lo
		rl	R5
		jr	C, loc_F1F
		ld	p01m, #92h	; Ports	0-1 mode
		swap	var_Z_lo
		pop	rp		; Register pointer
locret_F36:	ret
loc_F37:	ld	R12, #80h
		ld	R13, #0
loc_F3B:	lde	R2, @RR12
		cp	R2, R0
		jr	Z, loc_F4E
loc_F41:	and	R13, #0F0h
		add	R13, #10h
		adc	R12, #0
		jr	Z, loc_F7A
		jr	loc_F3B
loc_F4E:	inc	R13
		ld	R1, #3
loc_F51:	lde	R2, @RR12
		cp	R2, #95h
		jr	NZ, loc_F41
		inc	R13
		djnz	R1, loc_F51
		ld	R1, #4
		ld	R2, #18h
loc_F5F:	ldei	@R2, @RR12
		djnz	R1, loc_F5F
		ld	R2, R12
		ld	R3, R13
loc_F67:	lde	R1, @RR2
		add	R11, R1
		adc	R10, #0
		incw	RR2
		decw	RR8
		jr	NZ, loc_F67
		or	R10, R11
		jr	NZ, loc_F41
		call	@word_1C
loc_F7A:	jp	mon2

;------------------------------------------------------------------------------
; Monitor Move
; Maaaa aaaa aaaa verschiebt von Adresse aaaa nach aaaa aaaa Bytes
; im Speicher, auch bei berschneidung des Quell- und Zielbereichs
; erfolgt immer korrekte Verschiebung, also es wird automatisch
; festgestellt, ob das erste oder das letzte Byte zuerst verschoben
; werden mu, mon
;------------------------------------------------------------------------------

mon_move:	call	ADRE		
		ld	R8, R10
		ld	R9, R11
		call	ADRE		; Wandlung Adresse
		call	ATH16		; Ascii	To Hex 4 Ziffern
		jr	C, locret_F36
		cp	R8, R10
		jr	UGT, move4
		jr	C, move1
		cp	R9, R11
		jr	NC, move4
move1:		add	R9, R13
		adc	R8, R12
		add	R11, R13
		adc	R10, R12
move2:		decw	RR8
		decw	RR10
		lde	R0, @RR8
		lde	@RR10, R0
		decw	RR12
		jr	NZ, move2
		jr	loc_FB8
move4:		lde	R0, @RR8
		lde	@RR10, R0
		incw	RR8
		incw	RR10
		decw	RR12
		jr	NZ, move4
loc_FB8:	jp	PMON

;------------------------------------------------------------------------------
; Umrechnung Punkt-Koordinaten -> BWS-Pos
;------------------------------------------------------------------------------

sub_FBB:	cp	var_Y_lo, #0C0h
		jr	NC, loc_1000
		or	var_X_hi, var_X_hi
		jr	Z, loc_FCA
		cp	var_X_lo, #40h
		jr	NC, loc_1000
loc_FCA:	ld	R11, var_Y_lo	; pixtab muss 00-Adr. sein
		ld	R10, #hi(pixtab)
		ldc	R0, @RR10
		ld	R3, #0E0h
		and	R3, R0
		swap	R3
		rr	R3
		add	R3, #0C0h
		ld	R2, #0Bh
		ldc	R1, @RR2
		ld	R2, var_X_hi
		ld	R3, #19h
		ld	R10, var_X_lo
		rrc	R2
		rrc	R10
		rlc	R3
		rrc	R10
		rlc	R3
		rrc	R10
		rlc	R3
		add	R1, R10
		ld	R2, #0Bh
		ldc	R3, @RR2
		and	R0, #1Fh
		add	R0, #40h
		ret

;------------------------------------------------------------------------------
; Zeichensatz
; 1K, 000h bis 07Fh je 8 Byte
; 000h ist Code (nicht ndern im EPROM!)
;------------------------------------------------------------------------------

loc_1000:	
;;		ld	R0, #40h
;;		ld	R1, #79h
;;		ld	R3, #1
;;		ret

		include	es40_0800_zg.asm
		
;------------------------------------------------------------------------------
; 
;------------------------------------------------------------------------------

wr_block:	push	rp		; Register pointer
		srp	#60h
		ld	R1, #4
		ld	R2, #0Ch
loc_1408:	push	@unk_61
		inc	R1
		djnz	R2, loc_1408
		srp	#0
		ld	R11, byte_60
		ld	R4, #0F7h
		ld	R5, #0A4h
		lde	R13, @RR4
		ld	R5, #0A6h
		lde	R15, @RR4
		inc	R15
		lde	@RR4, R15
		ld	R4, #1Ch
		ld	R5, #0
wr_block1:	decw	word_4
		jr	NZ, wr_block1
		ld	R4, #0F5h
		ld	R5, #0FFh
		cp	byte_B,	#0FAh
		jr	NZ, wr_block2
		and	byte_D,	#7Fh
		lde	@RR4, R13
wr_block2:	ld	R5, #7Fh
		lde	@RR4, R11
		ld	R6, #0
		rcf
wr_block3:	lde	R7, @RR4
		adc	R6, R7
		inc	R5
		jr	NZ, wr_block3
		adc	byte_6,	#0
		ld	R5, #7Bh
		nop
		nop
		nop
		nop
		lde	@RR4, R15
		inc	R5
		lde	@RR4, R15
		inc	R5
		lde	@RR4, R6
		inc	R5
		lde	@RR4, R6
		ld	R5, #7Bh
		ld	tmr, #3		; Timer	mode
		ld	R3, #40h
		call	wait
		ld	R10, #40h
		ld	R3, #0
		ld	R6, #4
wr_block4:	call	wait
		djnz	R6, wr_block4
wr_block5:	lde	R6, @RR4
		ld	R7, #8
wr_block6:	rl	byte_6
		xor	R3, R10
		call	wait
		jr	NC, wr_block7
		call	wait
wr_block7:	xor	R3, R10
		call	wait
		jr	NC, wr_block8
		call	wait
wr_block8:	djnz	R7, wr_block6
		inc	R5
		jr	NZ, wr_block5
		xor	R3, R10
		call	wait
		ld	tmr, #43h	; Timer	mode
		srp	#60h
		ld	R1, #0Fh
		ld	R2, #0Ch
loc_1498:	pop	@unk_61
		dec	unk_61
		djnz	R2, loc_1498
		pop	rp		; Register pointer
		ret

		db 0FFh
		db 0FFh
		db 0FFh

;------------------------------------------------------------------------------
; 
;------------------------------------------------------------------------------

wait:		ld	byte_60, #18h
wait1:		dec	byte_60
		jr	NZ, wait1
		ret

		db 0FFh
		db 0FFh

;------------------------------------------------------------------------------
; 
;------------------------------------------------------------------------------

sub_14AE:	push	rp		; Register pointer
		srp	#60h
		ld	R1, #4
		ld	R2, #0Ch
loc_14B6:	push	@unk_61
		inc	R1
		djnz	R2, loc_14B6
		srp	#0
		ld	R4, #0F7h
		ld	R5, #0A6h
		lde	R6, @RR4
		inc	R6
		lde	@RR4, R6
		ld	byte_5E, R6
ld_block1:	ld	R4, #0F5h
		ld	R5, #7Bh
		ld	R10, #29h
		ld	R9, #21h
		ld	R6, #1
		and	R6, R3
ld_block2:	ld	R13, #1
		and	R13, R3
		cp	R13, R6
		jr	Z, ld_block2
ld_block3:	ld	R6, #1
		and	R6, R3
		ld	R7, #41h
ld_block4:	dec	byte_7
		jr	Z, ld_block3
		ld	R13, #1
		and	R13, R3
		cp	R13, R6
		jr	Z, ld_block4
		cp	R7, R9
		jr	NC, ld_block3
ld_block5:	ld	R8, #8
ld_block6:	ld	R7, #0
		ld	R6, #1
		and	R6, R3
ld_block7:	inc	R7
		ld	R13, #1
		and	R13, R3
		cp	R13, R6
		jr	Z, ld_block7
		ld	R6, #1
		and	R6, R3
ld_block8:	inc	R7
		ld	R13, #1
		and	R13, R3
		cp	R13, R6
		jr	Z, ld_block8
		cp	R10, R7
		rlc	word_8+1
		djnz	R8, ld_block6
		lde	@RR4, R9
		inc	R5
		jr	NZ, ld_block5
		ld	R5, #7Dh
		ld	R7, #0
		lde	R10, @RR4
		inc	R5
		lde	R9, @RR4
		inc	R5
		rcf
ld_block9:	lde	R6, @RR4
		adc	R7, R6
		inc	R5
		jr	NZ, ld_block9
		adc	byte_7,	#0
		cp	R7, R9
		jr	NZ, ld_block13
		cp	R7, R10
		jr	NZ, ld_block13
		ld	R5, #7Bh
		lde	R9, @RR4
		inc	R5
		lde	R10, @RR4
		nop
		nop
		cp	word_8+1, byte_5E
		jr	NZ, ld_block13
		xor	R9, R10
		jr	NZ, ld_block13
		ld	R5, #7Fh
		lde	R7, @RR4
		ld	R11, #0
		cp	byte_7,	#0FCh	; FC	    128	Datenbyte
		jr	Z, ld_block12
		ld	R5, #0FFh
		lde	R8, @RR4
		ld	R11, #80h
		or	R11, R8
		cp	byte_7,	#0FAh	; FA	    weniger als	128 Datenbyte
		jr	Z, ld_block12
		cp	byte_7,	#0FEh	; FE	    End	of File	Block
		jr	NZ, ld_block13
		ld	R14, #88h
ld_block10:	scf
ld_block11:	ld	tmr, #3		; Timer	mode
		ld	byte_13, R14
		jr	loc_1591
		db 0FFh
ld_block12:	nop
		nop
		rcf
		jr	ld_block11
ld_block13:	ld	tmr, #43h	; Timer	mode
ld_block14:	call	KEY
		and	byte_6D, #0DFh
		cp	byte_6D, #'B'
		jr	Z, ld_block16
		cp	byte_6D, #'C'
		jr	NZ, ld_block14
		jp	ld_block1
ld_block16:	ld	R14, #0FFh
		jr	ld_block10
loc_1591:	ld	R4, #0F7h
		ld	R5, #0A5h
		lde	@RR4, R11
		srp	#60h
		ld	R1, #0Fh
		ld	R2, #0Ch
loc_159D:	pop	@unk_61
		dec	unk_61
		djnz	R2, loc_159D
		pop	rp		; Register pointer
		ret

		db 0FFh
		db 0FFh

;------------------------------------------------------------------------------
; CLOSE schliet den Kanal wieder.
;------------------------------------------------------------------------------

tap_CLOSE:	push	rp		; Register pointer
		srp	#60h
		ld	R2, #0F7h
		ld	R3, #0A7h
		lde	R4, @RR2
		tm	unk_64,	#2
		jr	Z, loc_15CA
		ld	R3, #0A4h
		lde	R4, @RR2
		cp	unk_64,	#80h
		jr	Z, loc_15C5
		ld	R0, #0FAh
		call	wr_block
loc_15C5:	ld	R0, #0FEh
		call	wr_block
loc_15CA:	call	sub_910
		pop	rp		; Register pointer
		rcf
		ret

		db 0FFh

;------------------------------------------------------------------------------
; OPEN ffnet den Kanal fr nachfolgende PUT/GET. Der Zeiger auf
; einen eventuell notwandigen Dateinamen mu in Register %60/61
; stehen. In %15 ist der Modus zu bergeben:
; Bit 0 =1: ffnen zum Lesen (ber GET)
; Bit 1 =1: ffnen zum Schreiben (ber PUT)
; Bit 2-7 sollten 0 sein (evtl. spter genutzt)
;------------------------------------------------------------------------------

tap_OPEN:	push	rp		; Register pointer
		di
		srp	#0F0h
		ld	R8, #92h
		ld	R4, #64h
		ld	R5, #5
		ld	R1, #43h
		ld	R7, #8
		srp	#60h
		ld	R2, #0F7h
		ld	R3, #0A6h
		ld	R4, #0FFh
		lde	@RR2, R4
		inc	R3
		ld	R4, byte_15
		lde	@RR2, R4
		tm	byte_15, #2
		jr	NZ, loc_1602
		ld	tmr, #3		; Timer	mode
		ld	R4, #0
loc_15FA:	ld	R3, #0A4h
		lde	@RR2, R4
		pop	rp		; Register pointer
		rcf
		ret
loc_1602:	ld	R0, #0
		ld	R1, #0
loc_1606:	ld	R5, #18h
loc_1608:	djnz	R5, loc_1608
		decw	byte_60
		jr	NZ, loc_1606
		ld	R4, #80h
		jr	loc_15FA

		db 0FFh
		db 0FFh

;------------------------------------------------------------------------------
; PUT schreibt ein Byte aus %15 auf den Kanal
;------------------------------------------------------------------------------

tap_PUT:	push	rp		; Register pointer
		srp	#60h
		ld	R0, #0F7h
		ld	R1, #0A4h
		lde	R3, @RR0
		ld	R2, #0F5h
		ld	R4, #15h
		ldei	@RR2, @R4
		or	R3, R3
		jr	NZ, loc_1633
		ld	R0, #0FCh
		call	wr_block
		ld	R3, #80h
		ld	R0, #0F7h
		ld	R1, #0A4h
loc_1633:	lde	@RR0, R3
		rcf
		pop	rp		; Register pointer
		ret

		db 0FFh

;------------------------------------------------------------------------------
; PRISTRI druckt eine Zeichenkette ber CHAROUT aus, die nach dem
; CALL-Aufruf beginnt und mit einem %00-Kode endet. Die
; Programmbearbeitung wird hinter dem %00 fortgesetzt. (verndert
; Register %70-72)
;------------------------------------------------------------------------------

pristri0:	pop	unk_70
		pop	unk_71
		push	rp		; Register pointer
		srp	#70h
loc_1642:	ld	R2, #15h
		ldei	@R2, @RR0
		or	byte_15, byte_15
		jr	Z, loc_1650
		call	CHAROUT
		jr	loc_1642
loc_1650:	pop	rp		; Register pointer
		jp	@unk_70

		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh

;------------------------------------------------------------------------------
; 
;------------------------------------------------------------------------------

p_draw1:	ld	var_Y_lo, R7
		ld	var_X_hi, R4
		ld	var_X_lo, R5
		jp	PLOT
p_draw2:		ld	var_Y_lo, R5
		ld	var_X_hi, R6
		ld	var_X_lo, R7
		jp	PLOT
		
;------------------------------------------------------------------------------
; verbindet die Punkte (V,W) und (X,Y) durch eine Gerade der Farbe Z
;------------------------------------------------------------------------------

g_draw:		push	var_X_hi
		push	var_X_lo
		push	var_Y_lo
		push	rp		; Register pointer
		srp	#70h
		ld	R0, var_X_hi
		ld	R1, var_X_lo
		ld	R2, #0
		ld	R3, var_Y_lo
		ld	R4, var_V_hi
		ld	R5, var_V_lo
		ld	R6, #0
		ld	R7, var_W_lo
		ld	byte_5E, #16h	; hi p_Draw1
					;
		ld	byte_5F, #58h
		ld	R9, R5
		sub	R9, R1
		ld	R8, R4
		sbc	R8, R0
		jr	PL, draw1
		com	R9
		com	R8
		incw	RR8
draw1:		ld	R11, R7
		sub	R11, R3
		ld	R10, R6
		sbc	R10, R2
		jr	PL, draw2
		com	R11
		com	R10
		incw	RR10
draw2:		cp	R9, R11
		ld	R12, R8
		sbc	R12, R10
		jr	NC, loc_16C8
		ld	byte_5E, #16h	; hi p_draw2
		ld	byte_5F, #61h
		ld	R0, #0
		ld	R1, var_Y_lo
		ld	R2, var_X_hi
		ld	R3, var_X_lo
		ld	R4, #0
		ld	R5, var_W_lo
		ld	R6, var_V_hi
		ld	R7, var_V_lo
loc_16C8:	cp	R5, R1
		ld	R12, R4
		sbc	R12, R0
		jr	C, loc_16E8
		ld	R12, R1
		ld	R1, R5
		ld	R5, R12
		ld	R12, R0
		ld	R0, R4
		ld	R4, R12
		ld	R12, R3
		ld	R3, R7
		ld	R7, R12
		ld	R12, R2
		ld	R2, R6
		ld	R6, R12
loc_16E8:	sub	R1, R5
		sbc	R0, R4
		ld	R10, R0
		ld	R11, R1
		ld	R9, #0
		sub	R3, R7
		sbc	R2, R6
		jr	PL, loc_1700
		ld	R9, #0FFh
		com	R2
		com	R3
		incw	RR2
loc_1700:	ld	R12, #0
		ld	R13, #0
		ld	R14, #0
		ld	R15, #9
loc_1708:	sub	R3, R1
		sbc	R2, R0
		jr	NC, loc_1713
		add	R3, R1
		adc	R2, R0
		scf
loc_1713:	ccf
		rlc	R14
		rlc	R13
		rlc	R12
		rcf
		rlc	R3
		rlc	R2
		djnz	R15, loc_1708
		or	R9, R9
		jr	PL, loc_1730
		com	R14
		com	R13
		com	R12
		inc	R14
		jr	NZ, loc_1730
		incw	RR12
loc_1730:	ld	R8, #0
loc_1732:	call	@byte_5E
		incw	RR4
		add	R8, R14
		adc	R7, R13
		adc	R6, R12
		decw	RR10
		jr	PL, loc_1732
		pop	rp		; Register pointer
		pop	var_Y_lo
		pop	var_X_lo
		pop	var_X_hi
		ret

		db 0FFh

;------------------------------------------------------------------------------
; GET liest ein Byte von dem Kanal und liefert es in %13
;------------------------------------------------------------------------------

tap_GET:	push	rp		; Register pointer
		srp	#60h
		ld	R0, #0F7h
		ld	R1, #0A4h
		lde	R3, @RR0
		or	R3, R3
		jr	NZ, loc_1761
		call	sub_14AE
		jr	C, loc_1776
		ld	R3, #80h
		ld	R0, #0F7h
loc_1761:	ld	R1, #0A5h
		ld	R2, #0F5h
		ld	R4, #13h
		ldei	@R4, @RR2
		lde	R4, @RR0
		ld	R1, #0A4h
		cp	R3, R4
		jr	NZ, loc_1773
		ld	R3, #0
loc_1773:	lde	@RR0, R3
		rcf
loc_1776:	pop	rp		; Register pointer
		ret

		db 0FFh

;------------------------------------------------------------------------------
; LOAD ldt ab der in %20/21 stehenden Adresse die Daten von
; Kassette in den Speicher und gibt in %22/23 die Anzahl der
; fehlerfrei geladenen Byte zurck. Zustzlich steht in %24, ob ein
; nichtkorrigierbarer Fehler auftrat (%FF) (das wei man eigentlich
; sowieso, da man ja die B-Taste zum Abbruch drckte), oder alles ok
; war (%00). Die in %22/23 angegebene
; Bytezahl ist auch bei nichtkorrigierbaren Ladefehlern noch korrekt,
; SAVE und LOAD verndern die Register %13 und %15.
; Bei beiden besteht die Mglichkeit, einen Namen zu bergeben. Das
; wird durch die derzeit installierten Kassettenroutinen nicht
; genutzt, ist aber vielleicht spter fr einen Diskettenanschlu
; interessant. Dazu mu %60/61 auf das erste Byte des irgendwo im
; Speicher stehenden Namens zeigen.
;------------------------------------------------------------------------------

load0:		push	rp		; Register pointer
		srp	#20h
		ld	byte_15, #1
		call	0FFF0h
		jr	C, loc_179E
		ld	R2, #0
		ld	R3, #0
loc_178A:	call	0FFF6h
		jr	C, loc_1797
		ld	R4, #13h
		ldei	@RR0, @R4
		incw	unk_22
		jr	loc_178A
loc_1797:	ld	R4, #0
		cp	byte_13, #88h
		jr	Z, loc_17A0
loc_179E:	ld	R4, byte_13
loc_17A0:	call	0FFF3h
		pop	rp		; Register pointer
		ret

;------------------------------------------------------------------------------
; Druckerausgabe byte_15 auf Port 2, Serielles (Centronic)-Interface
; P26 Data, P27 Shift Left, P33 Busy, P25 /Strobe
;------------------------------------------------------------------------------

centr:		ld	p2, #30h	; Port 2 Bit 4 + Bit 5
		ld	p2m, #0Fh	; Port 2 mode
		push	rp		; Register pointer
		srp	#60h
		ld	R0, #8		; 8 Bit
		ld	R1, byte_15	; auszugebendes Zeichen
centr1:		ld	R2, #0
		rrc	unk_61		; R1
		rrc	unk_62		; R2
		or	unk_62,	#30h	; Bit 4 + Bit 5
		ld	R4, unk_62
		ld	p2, R2		; Port 2 (Strobe)
		ld	R3, #18h
centr2:		djnz	R3, centr2
		or	unk_62,	#40h	; Bit 6
		ld	p2, R2		; Port 2 (Data)
		ld	R3, #18h
centr3:		djnz	R3, centr3
		ld	p2, R4		; Port 2
		ld	R3, #18h
centr4:		djnz	R3, centr4
		djnz	R0, centr1	; next bit
		; wait while busy
centr5:		ld	R0, #8		; Bit 3
		and	byte_60, p3	; R0, Port 3 (Busy)
		jr	NZ, centr5
		; Strobe
		ld	p2, #10h	; Port 2 Bit 5 = 0
		ld	R3, #18h
centr6:		djnz	R3, centr6
		ld	p2, R4		; Port 2
		pop	rp		; Register pointer
		ret

;------------------------------------------------------------------------------
; BWKEY arbeitet genau wie WKEY (Beschreibung bei 13.), liefert
; das Zeichen aber in Register 05Ah
;------------------------------------------------------------------------------

p_bwkey:	push	byte_13
		call	WKEY
		ld	unk_5A,	byte_13
		pop	byte_13
		ret

;------------------------------------------------------------------------------
; Grafiksprungtabelle
;------------------------------------------------------------------------------

BWKEY:		jp	p_bwkey
DRAW:		jp	g_draw
PTEST:		jp	g_ptest
PLOT:		jp	g_plot

;------------------------------------------------------------------------------
; ??? 24 Werte
;------------------------------------------------------------------------------
; org xx00
m1800:		db  40h ; @
		db  41h ; A
		db  42h ; B
		db  44h ; D
		db  45h ; E
		db  46h ; F
		db  48h ; H
		db  49h ; I
		db  4Ah ; J
		db  4Ch ; L
		db  4Dh ; M
		db  4Eh ; N
		db  50h ; P
		db  51h ; Q
		db  52h ; R
		db  54h ; T
		db  55h ; U
		db  56h ; V
		db  58h ; X
		db  59h ; Y
		db  5Ah ; Z
		db  5Ch ; 
		db  5Dh ; ]
		db  5Eh ; ^
; zugehrige Werte
		db   0h ; @
		db  50h ; A
		db 0A8h ; B
		db   0h ; D
		db  50h ; E
		db 0A8h ; F
		db   0h ; H
		db  50h ; I
		db 0A8h ; J
		db   0h ; L
		db  50h ; M
		db 0A8h ; N
		db   0h ; P
		db  50h ; Q
		db 0A8h ; R
		db   0h ; T
		db  50h ; U
		db 0A8h ; V
		db   0h ; X
		db  50h ; Y
		db 0A8h ; Z
		db   0h ;  
		db  50h ; ]
		db 0A8h ; ^

;------------------------------------------------------------------------------
; 
;------------------------------------------------------------------------------

sub_1830:	ld	R0, #hi(m1800)
		ld	R1, byte_5C
		ldc	R4, @RR0
		add	R1, #24		; offs. Werte
		ldc	R5, @RR0
		add	unk_65,	cupos
		ret

		db 0FFh

;------------------------------------------------------------------------------
; 
;------------------------------------------------------------------------------

loc_1840:	add	R5, #28h
		jr	C, loc_184D
		jr	OV, loc_184E
		tcm	R5, #78h
		jr	NZ, locret_1854
		db  0Bh		; NOP mit nachfolgendem Befehl
loc_184D:	inc	R4
loc_184E:	add	R5, #8
		adc	R4, #0
locret_1854:	ret

		db 0FFh

;------------------------------------------------------------------------------
; 
;------------------------------------------------------------------------------

sub_1856:	ld	R1, byte_5C
		rl	R1
		rl	R1
		rl	R1
		ld	R0, #0
		ld	R2, R1
		rcf
		rlc	R1
		rlc	R0
		rlc	R1
		rlc	R0
		add	R1, R2
		adc	R0, #0FCh
		add	unk_61,	cupos
		adc	R0, #0
		ret

		db 0FFh

;------------------------------------------------------------------------------
; 
;------------------------------------------------------------------------------

sub_1878:	push	rp		; Register pointer
		srp	#60h
		call	sub_1856
		ld	R2, unk_5A
		lde	@RR0, R2
		ld	R3, #0
		ld	R1, #5
loc_1887:	rcf
		rrc	R2
		rrc	R3
		djnz	R1, loc_1887
		add	R2, R7
		call	sub_1830
		nop
		ld	R0, #8
		ld	R8, #0F7h
		ld	R9, #0A0h
		lde	R9, @RR8
		ld	R8, #60h
		ld	p01m, #0B2h	; Ports	0-1 mode
loc_18A1:	lde	@RR8, R9
		lde	R1, @RR2
		lde	@RR4, R1
		com	R1
		swap	R9
		lde	@RR8, R9
		lde	@RR4, R1
		swap	R9
		call	loc_1840
		inc	R3
		djnz	R0, loc_18A1
		ld	p01m, #92h	; Ports	0-1 mode
		pop	rp		; Register pointer
		ret

		db 0FFh

;------------------------------------------------------------------------------
; 
;------------------------------------------------------------------------------

sub_18BE:	ld	R3, #27h
		sub	unk_63,	cupos
		tm	byte_55, #40h
		jr	Z, loc_18D0
		tm	byte_5C, #1
		jr	NZ, loc_18D0
		add	unk_63,	#28h
loc_18D0:	ld	byte_5F, R3
		inc	byte_5F
		jr	sub_1856

		db 0FFh
		db 0FFh

;------------------------------------------------------------------------------
; 
;------------------------------------------------------------------------------

sub_18D8:	push	cupos
		push	byte_5C
loc_18DC:	call	sub_1856
		lde	R2, @RR0
		ld	unk_5A,	R2
		call	sub_1878
		inc	cupos
		cp	cupos, #28h
		jr	C, loc_18F1
		inc	byte_5C
		clr	cupos
loc_18F1:	decw	byte_5E
		jr	NZ, loc_18DC
		pop	byte_5C
		pop	cupos
		ret

;------------------------------------------------------------------------------
; 
;------------------------------------------------------------------------------

co:		tm	byte_55, #20h
		jr	Z, co1
		call	0F512h
co1:		ld	unk_5A,	byte_15
co2:		push	rp		; Register pointer
		srp	#50h
		or	R5, R5
		jr	PL, loc_1922
co3:		and	byte_55, #7Fh
		call	sub_1878
		inc	R11
		cp	cupos, #40
		jr	C, co4
		inc	R12
		ld	R11, #0
		jp	loc_1A9D
co4:		pop	rp		; Register pointer
		ret

loc_1922:	cp	unk_5A,	#10h
		jr	NC, co3

;------------------------------------------------------------------------------
; Control-Codes, Steuerkodes
; 00 nicht belegt
; 01 Kursor links
; 02 Kursor rechts
; 03 Kursor hoch
; 04 Kursor runter
; 05 HOM (home)
; 06 SOL (start of line)
; 07 DEL (delete)
; 08 DBS (delete back space)
; 09 INS (insert)
; 0A LDE (line delete)
; 0B LIN (line insert)
; 0C CLS (clear screen)
; 0D RET (return)
; 0E ESC (return)
; 0F nicht belegt
;------------------------------------------------------------------------------

; 01 Kursor links
ctrl01:		nop
		djnz	R10, ctrl02
		dec	cupos
		jr	PL, co4
		ld	R11, #27h
loc_1930:	dec	byte_5C
		jr	PL, co4
		ld	R12, #17h
		jr	co4

; 02 Kursor rechts
ctrl02:		djnz	R10, ctrl03
		inc	R11
		cp	cupos, #28h
		jr	C, co4
		ld	R11, #0
loc_1942:	inc	R12
		cp	byte_5C, #18h
		jr	C, co4
		ld	R12, #0
		jr	co4

; 03 Kursor hoch
ctrl03:		djnz	R10, ctrl04
		jr	loc_1930

; 04 Kursor runter
ctrl04:		djnz	R10, ctrl05
		jr	loc_1942

; 05 HOM (home)
ctrl05:		djnz	R10, ctrl06
loc_1956:	ld	R11, #0
		ld	R12, #0
		jr	co4

; 06 SOL (start of line)
ctrl06:		djnz	R10, ctrl07
		ld	R11, #0
		tm	byte_55, #40h
		jr	Z, co4
		and	byte_5C, #0FEh
		jr	co4

; 07 DEL (delete)
ctrl07:		djnz	R10, ctrl08
		srp	#60h
		call	sub_18BE
loc_1971:	dec	unk_63
		nop
		nop
		jr	MI, loc_1983
		incw	byte_60
		lde	R2, @RR0
		decw	byte_60
		lde	@RR0, R2
		incw	byte_60
		jr	loc_1971
loc_1983:	ld	R2, #20h
		lde	@RR0, R2
		clr	byte_5E
		call	sub_18D8
loc_198C:	jr	co4

; 08 DBS (delete back space)
ctrl08:		djnz	R10, ctrl09
		or	R11, R11
		jr	NZ, loc_199E
		tm	byte_55, #40h
		jr	Z, co4
		tm	byte_5C, #1
		jr	Z, co4
loc_199E:	ld	R10, #1
		call	co2
		ld	R10, #20h
		call	co2
		ld	R10, #1
		call	co2
loc_19AD:	jr	loc_198C

; 09 INS (insert)
ctrl09:		djnz	R10, ctrl0a
		srp	#60h
		call	sub_18BE
		inc	R3
		ld	R2, #20h
loc_19B9:		lde	R4, @RR0
		lde	@RR0, R2
		ld	R2, unk_64
		incw	byte_60
		djnz	R3, loc_19B9
		clr	byte_5E
		call	sub_18D8
loc_19C8:	jr	loc_19AD

; 0A LDE (line delete)
ctrl0a:		djnz	R10, ctrl0b
		ld	R10, #6
		call	co2
		srp	#60h
		call	sub_1856
		ld	R2, #0
		ld	R3, #28h
		tm	byte_55, #40h
		jr	Z, loc_19E1
		ld	R3, #50h
loc_19E1:	ld	R9, unk_63
		ld	byte_5F, R3
		ld	R4, #0FFh
		ld	R5, #0C0h
		add	R3, R1
		adc	R2, R0
		sub	R5, R3
		sbc	R4, R2
		incw	unk_64
		decw	unk_64
		jr	Z, loc_1A0B
		clr	byte_5E
		add	byte_5F, unk_65
		adc	byte_5E, unk_64
loc_19FF:	lde	R8, @RR2
		lde	@RR0, R8
		incw	unk_62
		incw	byte_60
		decw	unk_64
		jr	NZ, loc_19FF
loc_1A0B:	ld	R2, #20h
loc_1A0D:	lde	@RR0, R2
		incw	byte_60
		djnz	R9, loc_1A0D
		call	sub_18D8
loc_1A16:	jr	loc_19C8

; 0B LIN (line insert)
ctrl0b:		djnz	R10, ctrl0c
		nop
		ld	R10, #6
		call	co2
		srp	#60h
		call	sub_1856
		ld	R2, #0FFh
		ld	R3, #0BFh
		ld	R9, #28h
		tm	byte_55, #40h
		jr	Z, loc_1A32
		ld	R9, #50h
loc_1A32:	sub	R3, R9
		ld	byte_5E, #0FFh
		ld	byte_5F, #0C0h
		sub	byte_5F, unk_61
		sbc	byte_5E, byte_60
		ld	R4, byte_5E
		ld	R5, byte_5F
		sub	R5, R9
		sbc	unk_64,	#0
		incw	unk_64
		decw	unk_64
		jr	Z, loc_1A5F
		ld	R0, #0FFh
		ld	R1, #0BFh
loc_1A53:	lde	R8, @RR2
		lde	@RR0, R8
		decw	byte_60
		decw	unk_62
		decw	unk_64
		jr	NZ, loc_1A53
loc_1A5F:	ld	R2, #20h
loc_1A61:	lde	@RR0, R2
		decw	byte_60
		djnz	R9, loc_1A61
loc_1A67:	call	sub_18D8
		jr	loc_1A16

; 0C CLS (clear screen)
ctrl0c:		djnz	R10, ctrl0d
		ld	R9, #20h
		ld	R14, #0FCh
		ld	R15, #0
		ld	R10, #3
		ld	R11, #0C0h
loc_1A78:	lde	@RR14, R9
		incw	byte_5E
		decw	unk_5A
		jr	NZ, loc_1A78
		ld	R14, #3
		ld	R15, #0C0h
		srp	#60h
		clr	byte_5C
		jr	loc_1A67
loc_1A8A:	jr	loc_1A16

; 0D RET (return)
ctrl0d:		djnz	R10, ctrl0e
		ld	R10, #6
		call	co2
		inc	R12
		tm	byte_55, #40h
		jr	Z, loc_1A9D
		inc	R12
		and	byte_5C, #0FEh
loc_1A9D:	cp	byte_5C, #18h
		jr	C, loc_1A8A
		tm	byte_55, #10h
		jp	Z, loc_1956
		srp	#60h
		ld	R0, #0FCh
		ld	R1, #0
		ld	R2, #0FCh
		ld	R3, #0A0h
		nop
		ld	R4, #3
		ld	R5, #0C0h
		ld	byte_5E, R4
		ld	byte_5F, R5
		sub	R5, R3
		ld	R9, unk_63
		nop
loc_1AC0:	lde	R8, @RR2
		lde	@RR0, R8
		incw	byte_60
		incw	unk_62
		decw	unk_64
		jr	NZ, loc_1AC0
		ld	R8, #20h
loc_1ACE:	lde	@RR0, R8
		incw	byte_60
		djnz	R9, loc_1ACE
		clr	byte_5C
		call	sub_18D8
		ld	byte_5C, #14h
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		jr	loc_1A8A

; 0E ESC (escape)
ctrl0e:		djnz	R10, ctrl0f
		or	byte_55, #80h

; 0F nicht belegt
ctrl0f:		jr	loc_1A8A

		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh

;------------------------------------------------------------------------------
; 
;------------------------------------------------------------------------------

sub_1AF0:	push	rp		; Register pointer
		srp	#60h
		push	R1
		push	R0
		ld	R1, #10h
loc_1AFA:	xor	p3, #40h	; Port 3
		ld	R0, #60h
loc_1AFF:	djnz	R0, loc_1AFF
		djnz	R1, loc_1AFA
		pop	R0
		pop	R1
		pop	rp		; Register pointer
		ret

;------------------------------------------------------------------------------
; 
;------------------------------------------------------------------------------

key0:		push	rp		; Register pointer
		srp	#60h
		ld	R13, byte_55
		swap	byte_6D
		and	byte_6D, #80h
		ld	R0, #0
		ld	R2, #0
		ld	R5, #0Fh
loc_1B1B:	ld	R4, #7Fh
		ldc	R3, @RR4
		push	R5
		ld	R1, #4		; ANZAHL TASTATURZEILEN 
loc_1B23:	rlc	R3
		jr	NC, loc_1B37
		ld	R4, #hi(keytab)
		lde	R8, @RR4
		cp	R8, #0B0h
		jr	NC, loc_1B35
		ld	R2, #80h
		or	R2, R5
		db  0Dh		; nop -	code
loc_1B35:	ld	R0, R8
loc_1B37:	add	R5, #10h
		djnz	R1, loc_1B23
		nop
		pop	R5
		dec	R5
		jr	PL, loc_1B1B
		xor	R2, #80h
		jr	PL, loc_1B4D
loc_1B48:	ld	R13, #0
loc_1B4A:	pop	rp		; Register pointer
		ret
loc_1B4D:	ld	R1, #0
		rrc	R0
		rrc	R1
		rrc	R0
		rrc	R1
		and	R0, #0Fh
		add	R0, #hi(keytab)	; 1Dh
		or	R1, R2
		lde	R3, @RR0
		or	R3, R3
		jr	MI, loc_1B7B
		tm	R12, #20h
		jr	Z, loc_1B77
		cp	R3, #61h
		jr	C, loc_1B77
		cp	R3, #7Bh
		jr	NC, loc_1B77
		sub	R3, #20h
loc_1B77:	or	R13, R3
		jr	loc_1B4A
loc_1B7B:	rcf
		rlc	R3
		rcf
		rlc	R3
		or	R3, #0E0h
		ld	R2, #0F7h
		call	@unk_62
		srp	#60h
		ld	R0, #30h
loc_1B8C:	call	sub_1AF0
		djnz	R0, loc_1B8C
		jr	loc_1B48
		
		db 0FFh

;------------------------------------------------------------------------------
; 
;------------------------------------------------------------------------------

wkey0:		push	rp		; Register pointer
		srp	#60h
		ld	R0, #0F7h
		ld	R1, #0A0h
		lde	R2, @RR0
		push	unk_62
		inc	R1
		lde	R2, @RR0
		dec	unk_61
		lde	@RR0, R2
		clr	byte_5E
		ld	byte_5F, #1
		call	sub_18D8
		pop	unk_62
		ld	R0, #0F7h
		ld	R1, #0A0h
		lde	@RR0, R2
		jr	loc_1BBC
loc_1BB9:	and	byte_6C, #7Fh
loc_1BBC:	ld	byte_5F, R13
		call	KEY
		ld	byte_13, R13
		or	R13, R13
		jr	Z, loc_1BB9
		ld	R0, #1Fh
		and	R0, R12
		ld	R15, byte_60
loc_1BCD:	or	R15, R15
		jr	NZ, loc_1BCD
		cp	byte_5F, byte_6D
		jr	Z, loc_1BDB
		and	byte_6C, #7Fh
		jr	loc_1BF4
loc_1BDB:	or	R12, R12
		jr	MI, loc_1BF4
		ld	R15, #80h
loc_1BE1:	call	KEY
		or	R13, R13
		jr	Z, loc_1BB9
		cp	byte_5F, byte_6D
		jr	NZ, loc_1BB9
		or	R15, R15
		jr	NZ, loc_1BE1
		or	byte_6C, #80h
loc_1BF4:	tm	byte_6C, #40h
		jr	Z, loc_1BFC
		call	sub_1AF0
loc_1BFC:	clr	byte_5E
loc_1BFE:	ld	byte_5F, #1
		call	sub_18D8
		pop	rp		; Register pointer
		ret

		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh

;------------------------------------------------------------------------------
; Durch den IRQ wird das Register %6F alle 0.004 s
; heruntergezhlt, bis es auf 0 steht. Dies dient in der WKEY-Routine
; zur Zeitmessung. Ohne laufenden IRQ bleibt WKEY hngen.
;------------------------------------------------------------------------------

p_irq1:		or	byte_6F, byte_6F
		jr	Z, locret_1C15
		dec	byte_6F
locret_1C15:	iret

		db 0FFh
		db 0FFh
		db 0FFh

;------------------------------------------------------------------------------
; 
;------------------------------------------------------------------------------

CHARIN0:	push	rp		; Register pointer
		srp	#50h
		or	R8, R8
		jr	MI, ci4
ci1:		dec	byte_58
		jr	PL, ci3
		ld	R10, #0Dh
ci2:		ld	byte_13, R10
		pop	rp		; Register pointer
		ret
ci3:		ld	R14, #0F7h
		ld	R15, #0A2h
		lde	R10, @RR14
		push	unk_5A
		inc	R10
		lde	@RR14, R10
		pop	byte_5F
		lde	R10, @RR14
		jr	ci2
ci4:		call	WKEY
		ld	R10, byte_13
		call	co2
		tm	byte_55, #80h
		jr	NZ, ci4
		cp	byte_13, #0Dh
		jr	NZ, ci4
		srp	#60h
		call	sub_1856
		ld	R2, #28h
		tm	byte_55, #40h
		jr	Z, loc_1C5D
		ld	R2, #50h
loc_1C5D:	sub	R1, R2
		sbc	R0, #0
		ld	R4, #0F7h
		ld	R5, #0
		clr	byte_58
loc_1C68:	lde	R3, @RR0
		lde	@RR4, R3
		cp	R3, #20h
		jr	Z, loc_1C75
		ld	byte_58, R5
		inc	byte_58
loc_1C75:	incw	byte_60
		inc	R5
		djnz	R2, loc_1C68
		srp	#50h
		ld	R14, #0F7h
		ld	R15, #0A2h
		ld	R10, #0
		lde	@RR14, R10
		jr	ci1

;------------------------------------------------------------------------------
; SAVE speichert den Speicherbereich auf Kassette, dessen
; Anfangsadresse in %20/21 steht mit der in %22/23 angegebenen Anzahl
; von Bytes ab. Fehlerrckmeldung wie LOAD.
;------------------------------------------------------------------------------

save0:		push	rp		; Register pointer
		srp	#20h
		ld	byte_15, #2
		call	0FFF0h
		jr	C, loc_1CA2
loc_1C92:	ld	R4, #15h
		ldei	@R4, @RR0
		call	0FFF9h
		jr	C, loc_1CA2
		decw	unk_22
		jr	NZ, loc_1C92
		ld	R4, #0
		db  0Dh
loc_1CA2:	ld	R4, byte_13
		call	0FFF3h
		jr	NC, loc_1CAB
		ld	R4, byte_13
loc_1CAB:	pop	rp		; Register pointer
		ret

;------------------------------------------------------------------------------
; RND liefert nach jedem Aufruf in %74/75 eine 16-Bit-
; Zufallszahl. Diese kann man auch aus %F7A8/9 nehmen. Nutzt %70-77
;------------------------------------------------------------------------------

rnd0:		push	rp		; Register pointer
		srp	#70h
		ld	R0, #0F7h
		ld	R1, #0A8h
		ld	R2, #74h
		ld	R3, #4
loc_1CBA:	ldei	@R2, @RR0
		djnz	R3, loc_1CBA
		ld	R3, #3
		and	R3, R7
		add	R5, R7
		adc	R4, R6
		incw	RR6
		djnz	R3, loc_1CD3
		rcf
		tm	R4, #10h
		jr	Z, loc_1CD1
		scf
loc_1CD1:	jr	loc_1CEF
loc_1CD3:	djnz	R3, loc_1CDE
		scf
		tm	R4, #80h
		jr	Z, loc_1CD1
		rcf
		jr	loc_1CD1
loc_1CDE:	djnz	R3, loc_1CE9
		rcf
		tm	R5, #4
		jr	Z, loc_1CD1
		scf
		jr	loc_1CD1
loc_1CE9:	scf
		or	R5, R5
		jr	PL, loc_1CEF
		rcf
loc_1CEF:	rlc	byte_74
		rlc	byte_75
		ld	R2, #74h
		ld	R1, #0A8h
		ld	R3, #4
loc_1CF9:	ldei	@RR0, @R2
		djnz	R3, loc_1CF9
		pop	rp		; Register pointer
		ret

;------------------------------------------------------------------------------
; Tastaturbelegung
;------------------------------------------------------------------------------

		;org	1D00h
		align 100h

; 4-Zeilen-Tastatur:
; auf %1B22: %04
; max. 6 Ebenen zu je 64 Byte
; Grundebene %1D00-1D3F
; SHT1 %C1 %1D40-1D7F
; SHT2 %C2 %1D80-1DBF
; SHT3 %C3 %1DC0-1DFF
; SHT4 %C4 %1E00-1E3F
; SHT5 %C5 %1E40-1E7F

; 5- bis 8-Zeilen-Tastatur:
; auf %1B22: %05 bis %08 (Zeilenzahl)
; max. 3 Ebenen zu je 128 Byte
; Grundebene %1D00-1D7F
; SHT1 %C2 %1D80-1DFF
; SHT2 %C4 %1E00-1E7F
; 
keytab:
; Grundebene %1D00-1D3F
		db 0FFh, 0FFh, '1', '2', '3', '4', '5', '6', '7', '8', '9', '0', '<', 0FFh, 0FFh, 0FFh
		db 0FFh, 0C3h, 'q', 'w', 'e', 'r', 't', 'y', 'u', 'i', 'o', 'p', '+', 0FFh, 0FFh, 0FFh
		db 0FFh, 0C2h, 'a', 's', 'd', 'f', 'g', 'h', 'j', 'k', 'l', ';', '*', 0FFh, 0FFh, 0FFh
		db 0FFh, 0C1h, 'z', 'x', 'c', 'v', 'b', 'n', 'm', ',', '.', ' ', 0Dh, 0FFh, 0FFh, 0FFh
; SHT1 %C1 %1D40-1D7F
		db 0FFh, 0FFh, '!', '"', '#', '$', '%', '&', 27h, '@', '(', ')', '>', 0FFh, 0FFh, 0FFh
		db 0FFh, 0FFh, 'Q', 'W', 'E', 'R', 'T', 'Y', 'U', 'I', 'O', 'P', '-', 0FFh, 0FFh, 0FFh
		db 0FFh, 0FFh, 'A', 'S', 'D', 'F', 'G', 'H', 'J', 'K', 'L', ':', '/', 0FFh, 0FFh, 0FFh
		db 0FFh, 0FFh, 'Z', 'X', 'C', 'V', 'B', 'N', 'M', '[', ']', '=', '?', 0FFh, 0FFh, 0FFh
; SHT2 %C2 %1D80-1DBF
		db 0FFh, 0FFh, 8, 9, 0Ch, 0FFh, 1Ah, 1Bh, 1Ch, 1Dh, 1Eh, 1Fh, 5Eh, 0FFh, 0FFh, 0FFh
		db 0FFh, 0FFh, 7, 3, 6, 0Ah, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 5Ch, 0FFh, 0FFh, 0FFh
		db 0FFh, 0FFh, 1, 5, 2, 0Bh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 7Ch, 0FFh, 0FFh, 0FFh
		db 0FFh, 0FFh, 0Fh, 4, 0FFh, 0Eh, 0FFh, 0FFh, 0FFh, 7Bh, 7Dh, 5Fh, 7Eh, 0FFh, 0FFh, 0FFh
; SHT3 %C3 %1DC0-1DFF
		db 0FFh, 0FFh, 11h, 12h, 13h, 14h, 15h, 16h, 17h, 18h, 19h, 10h, 7Fh, 0FFh, 0FFh, 0FFh
		db 0FFh, 0FFh, 80h, 81h, 82h, 83h, 84h, 85h, 86h, 87h, 0FFh, 0FFh, 60h, 0FFh, 0FFh, 0FFh
		db 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh
		db 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh
; SHT4 %C4 %1E00-1E3F
		db 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh
		db 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh
		db 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh
		db 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh
; SHT5 %C5 %1E40-1E7F
		db 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh
		db 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh
		db 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh
		db 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh, 0FFh


;------------------------------------------------------------------------------
; 
;------------------------------------------------------------------------------

loc_1E80:	push	rp		; Register pointer
		srp	#60h
		ld	R0, #0FCh
		ld	R1, #0
		ld	R2, #3
		ld	R3, #0C0h
		ld	byte_5E, R2
		ld	byte_5F, R3
loc_1E90:	lde	R4, @RR0
		sub	R0, #4
		lde	R5, @RR0
		lde	@RR0, R4
		add	R0, #4
		lde	@RR0, R5
		incw	RR0
		decw	RR2
		jr	NZ, loc_1E90
		clr	cupos
		clr	byte_5C
		call	sub_18D8
		pop	rp		; Register pointer
		ret

		db 0FFh
		db 0FFh

; Funktionstasten
; F1 Tastaturpiep umschalten (an/aus)
; F2 Caps umschalten (klein-)gro Buchstabenwandlung an/aus bei Tastatureingabe)
; F3 Drucker an
; F4 Drucker aus
; F5 Bildschirm wechseln. Danach ist der auf HOM-Position stehende
; Kursor bis zum nchsten Tastendruck unsichtbar. Der Wechsel
; erfolgt zwischen zwei Textbildschirmen durch Speicherverschiebung.
; F6 Tastencodeverschiebung umschalten (an/aus).

xxx_1EB0:	xor	byte_6C, #40h
		ret

xxx_1EB4:	xor	byte_6C, #20h
		ret

xxx_1EB8:	or	byte_55, #20h
		ret

xxx_1EBC:	and	byte_55, #0DFh
		ret

xxx_1EC0:	jp	loc_1E80
		ret

xxx_1EC4:	xor	byte_55, #8
		ret

xxx_1EC8:	db 0AFh ; 
		db 0AFh ; 
		db 0AFh ; 
		db 0AFh ; 

xxx_1ECC:	db 0AFh ; 
		db 0AFh ; 
		db 0AFh ; 
		db 0AFh ; 

;------------------------------------------------------------------------------
; Monitor Hex>Dez
; %aaaa gibt die Hexzahl dezimal aus
;------------------------------------------------------------------------------

mon_hexdez:	call	ADRE		
		ld	R5, #'#'
		call	CHAROUT
		ld	R12, #0
		ld	R8, #0
		ld	R9, #0
		ld	R13, #10h
loc_1EE0:	rlc	unk_1B
		rlc	unk_1A
		adc	R9, R9
		da	byte_19
		adc	R8, R8
		da	byte_18
		adc	R12, R12
		djnz	R13, loc_1EE0
		ld	R5, #'0'
		add	R5, R12
		call	CHAROUT
		call	HTA16		; druckt 018h/019h als 4-Ziffer-Hexzahl aus
		jp	PRRET		; druckt einen Return-Kode aus (00Dh)

		db 0FFh
		db 0FFh
		db 0FFh

;------------------------------------------------------------------------------
; 
;------------------------------------------------------------------------------

showplayer0:	push	rp		; Register pointer
		srp	#60h
		call	sub_FBB
		ld	R4, byte_60
		ld	R5, unk_61
		srp	#70h
		ld	R0, #60h
		ld	R1, var_V_lo
		com	unk_71
		swap	unk_71
		lde	@RR0, R1
		ld	R0, var_W_hi
		ld	R1, var_W_lo
		ld	R2, var_Z_hi
		ld	R3, var_Z_lo
		ld	R12, #10h
loc_1F21:	ld	R4, unk_63
		ld	R5, #76h
		ldei	@R5, @RR0
		ldei	@R5, @RR0
		ldei	@R5, @RR0
loc_1F2B:	rl	R4
		jr	C, loc_1F37
		rrc	byte_76
		rrc	byte_77
		rrc	byte_78
		jr	loc_1F2B
loc_1F37:	ld	R10, unk_64
		ld	R11, unk_65
		ld	R5, #76h
		ld	R4, #3
		ld	p01m, #0B2h	; Ports	0-1 mode
loc_1F42:	tm	p3, #4		; Port 3
		jr	NZ, loc_1F42
loc_1F47:	lde	R9, @RR10
		lde	@RR2, R9
		xor	R9, @R5
		tm	var_V_hi, #1
		jr	Z, loc_1F54
		or	R9, @R5
loc_1F54:	lde	@RR10, R9
		inc	R5
		incw	RR2
		incw	RR10
		djnz	R4, loc_1F47
		ld	p01m, #92h	; Ports	0-1 mode
		srp	#60h
		call	loc_1840
		srp	#70h
		djnz	R12, loc_1F21
		pop	rp		; Register pointer
		ret

		db 0FFh

;------------------------------------------------------------------------------
; 
;------------------------------------------------------------------------------

hideplayer0:	push	rp		; Register pointer
		srp	#60h
		call	sub_FBB
		ld	R2, #60h
		ld	R3, var_V_lo
		com	unk_63
		swap	unk_63
		lde	@RR2, R3
		ld	R2, var_Z_hi
		ld	R3, var_Z_lo
		ld	R9, #10h
		ld	R4, byte_60
		ld	R5, unk_61
loc_1F88:	ld	R8, #3
		ld	p01m, #0B2h	; Ports	0-1 mode
loc_1F8D:	tm	p3, #4		; Port 3
		jr	NZ, loc_1F8D
loc_1F92:	lde	R6, @RR2
		lde	@RR0, R6
		incw	RR0
		incw	RR2
		djnz	R8, loc_1F92
		ld	p01m, #92h	; Ports	0-1 mode
		call	loc_1840
		ld	R0, unk_64
		ld	R1, unk_65
		djnz	R9, loc_1F88
		pop	rp		; Register pointer
		ret

		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh
		db 0FFh

;------------------------------------------------------------------------------
; Block	wird in	BEGIN nach FFE0	kopiert
;------------------------------------------------------------------------------

kanal_sv:

; Kanal 2
		phase	0FFE0h

k2_open:	nop			; OPEN
		nop
		nop
k2_close:	nop			; CLOSE
		rcf
		ret
k2_get:		jp	lCHARIN		; GET
k2_put:		jp	lCHAROUT	; PUT
k2_special:	rcf			; SPECIAL
		nop
		ret
		db 0FFh
; Kanal 3
k3_open:	jp	tap_OPEN
k3_close:	jp	tap_CLOSE
k3_get:		jp	tap_GET
k3_put:		jp	tap_PUT
k3_special:	rcf			; SPECIAL
		nop
		ret
		db 0FFh

		dephase

		end

;------------------------------------------------------------------------------
; 
;------------------------------------------------------------------------------
