; JU+TE TINY ES4.0
; Schreibmaschine 3004 als Drucker
; s. JU+TE 5/1989, Seiten 376-380

; reass. V.Pohlers 05.01.2022


		cpu	z8601
		assume RP:0C0h		; prevent register optimization thru AS

; AS-Funktionen
hi	function x,(x>>8)&255
lo	function x, x&255
;



p3 equ	3
imr equ 0FBh				; Interrupt mask register
rp equ	0FDh				; Register pointer

byte_15	equ 15h

	 org 0F480h

; Codetabelle S3004
; Position des Zeichens auf dem Typenrad der Schreibmaschine         
; s.a. http://erika-electronic.de/54,0,typenraederuebersicht,index,0.html

codtab:		db   0  ;  0
		db  72h ;  1 <-
		db  71h ;  2 ->
		db  76h ;  3 hoch
		db  75h ;  4 runter
		db  77h ;  5 HOM -> NL
		db  78h ;  6 SOL -> CR
		db   0  ;  7 DEL
		db   0  ;  8 DBS
		db   0  ;  9 INS
		db   0  ;  A LDE
		db   0  ;  B LIN
		db  77h ;  C CLS -> NL
		db  77h ;  D RET -> NL
		db  45h ;  E ESC
		db   0  ;  F
		db  0Dh ; 10
		db  11h ; 11
		db  10h ; 12
		db  0Fh ; 13
		db  0Eh ; 14
		db  0Ch ; 15
		db  0Bh ; 16
		db  0Ah ; 17
		db   9  ; 18
		db   8  ; 19 
		db  65h ; 1A 
		db  66h ; 1B 
		db  67h ; 1C 
		db  3Fh ; 1D 
		db  3Ch ; 1E 
		db  3Ah ; 1F 
		;
		db  71h ; 20 SP
		db  42h ; 21 !
		db  43h ; 22 "
		db  41h ; 23 #
		db  48h ; 24 $
		db   4  ; 25 %
		db   2  ; 26 &
		db  17h ; 27 '
		db  1Dh ; 28 (
		db  1Fh ; 29 )
		db  1Bh ; 2A *
		db  25h ; 2B +
		db  64h ; 2C ,
		db  62h ; 2D -
		db  63h ; 2E .
		db  40h ; 2F /
		db  0Dh ; 30 0
		db  11h ; 31 1
		db  10h ; 32 2
		db  0Fh ; 33 3
		db  0Eh ; 34 4
		db  0Ch ; 35 5
		db  0Bh ; 36 6
		db  0Ah ; 37 7
		db   9  ; 38 8
		db   8  ; 39 9
		db  13h ; 3A :
		db  3Bh ; 3B ;
		db  71h ; 3C <
		db  2Eh ; 3D =
		db  71h ; 3E >
		db  35h ; 3F ?
		db  44h ; 40 @
		db  30h ; 41 A
		db  18h ; 42 B
		db  20h ; 43 C
		db  14h ; 44 D
		db  34h ; 45 E
		db  3Eh ; 46 F
		db  1Ch ; 47 G
		db  12h ; 48 H
		db  21h ; 49 I
		db  32h ; 4A J
		db  24h ; 4B K
		db  2Ch ; 4C L
		db  16h ; 4D M
		db  2Ah ; 4E N
		db  1Eh ; 4F O
		db  2Fh ; 50 P
		db  1Ah ; 51 Q
		db  36h ; 52 R
		db  33h ; 53 S
		db  37h ; 54 T
		db  28h ; 55 U
		db  22h ; 56 V
		db  2Dh ; 57 W
		db  26h ; 58 X
		db  31h ; 59 Y
		db  38h ; 5A Z
		db  27h ; 5B [
		db  46h ; 5C
		db  27h ; 5D ]
		db  19h ; 5E ^
		db   1  ; 5F _
		;
		db   6  ; 60 RAUTE
		db  61h ; 61 a
		db  4Eh ; 62 b
		db  57h ; 63 c
		db  53h ; 64 d
		db  5Ah ; 65 e
		db  49h ; 66 f
		db  60h ; 67 g
		db  55h ; 68 h
		db   5  ; 69 i
		db  4Bh ; 6A j
		db  50h ; 6B k
		db  4Dh ; 6C l
		db  4Ah ; 6D m
		db  5Ch ; 6E n
		db  5Eh ; 6F o
		db  5Bh ; 70 p
		db  52h ; 71 q
		db  59h ; 72 r
		db  58h ; 73 s
		db  56h ; 74 t
		db  5Dh ; 75 u
		db  4Fh ; 76 v
		db  4Ch ; 77 w
		db  5Fh ; 78 x
		db  51h ; 79 y
		db  54h ; 7A z
		db  15h ; 7B {
		db  27h ; 7C |
		db  23h ; 7D }
		db   7  ; 7E ~
		db  47h ; 7F 

		; org 0F500h
		db   0  ;
		db  64h ;
		db   0  ;
		db  83h ;
		db   0  ;
		db 0B9h ;
		db   0  ;
		db   7  ;
		db   0  ;
		db   1  ;
		db   0  ;
		db  79h ;
		db   0  ;
		db   3  ;
		db   0  ;
		db  11h ;
		db   0  ;
		db  84h ;

		; org 0F512h
start:		push	rp		; Register pointer
		srp	#60h
		ld	R0, #hi(codtab)
		ld	R1, byte_15
		or	R1, #lo(codtab)

; s.a. JU+TE 10/1989, Seiten 790-793
; software seriell

DRU:		tm	p3, #2		; Port 3
		jr	NZ, DRU
		djnz	R3, DRU
		lde	R1, @RR0	; Datenregister
		ld	R3, #0Bh	; Bitzhler
		push	imr		; Interrupt mask register
		di
		; Startbit (Nullbit) ausgeben
DR5:		and	p3, #0DFh	; 0-Pegel
DR6:		ld	R0, #0B6h	; Zeitzhler
DR2:		nop
		djnz	R0, DR2		; abwarten
		djnz	R3, DR1		; nchstes Bit   
		; Ende
		pop	imr		; Interrupt mask register
		pop	rp		; Register pointer
		ret
DR1:		cp	R3, #3		; am Ende 2 Stoppbits
		jr	C, DR4
		rr	R1		; zuvor die acht Datenbits ausgeben
		jr	NC, DR5
DR4:		or	p3, #20h	; 1-Pegel
		jr	DR6

		end
